/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: storage_resolve.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "storage_resolve.pb-c.h"
void   spotify__download__proto__storage_resolve_response__init
                     (Spotify__Download__Proto__StorageResolveResponse         *message)
{
  static const Spotify__Download__Proto__StorageResolveResponse init_value = SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__INIT;
  *message = init_value;
}
size_t spotify__download__proto__storage_resolve_response__get_packed_size
                     (const Spotify__Download__Proto__StorageResolveResponse *message)
{
  assert(message->base.descriptor == &spotify__download__proto__storage_resolve_response__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__download__proto__storage_resolve_response__pack
                     (const Spotify__Download__Proto__StorageResolveResponse *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__download__proto__storage_resolve_response__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__download__proto__storage_resolve_response__pack_to_buffer
                     (const Spotify__Download__Proto__StorageResolveResponse *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__download__proto__storage_resolve_response__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Download__Proto__StorageResolveResponse *
       spotify__download__proto__storage_resolve_response__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Download__Proto__StorageResolveResponse *)
     protobuf_c_message_unpack (&spotify__download__proto__storage_resolve_response__descriptor,
                                allocator, len, data);
}
void   spotify__download__proto__storage_resolve_response__free_unpacked
                     (Spotify__Download__Proto__StorageResolveResponse *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__download__proto__storage_resolve_response__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCEnumValue spotify__download__proto__storage_resolve_response__result__enum_values_by_number[3] =
{
  { "CDN", "SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__CDN", 0 },
  { "STORAGE", "SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__STORAGE", 1 },
  { "RESTRICTED", "SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__RESTRICTED", 3 },
};
static const ProtobufCIntRange spotify__download__proto__storage_resolve_response__result__value_ranges[] = {
{0, 0},{3, 2},{0, 3}
};
static const ProtobufCEnumValueIndex spotify__download__proto__storage_resolve_response__result__enum_values_by_name[3] =
{
  { "CDN", 0 },
  { "RESTRICTED", 2 },
  { "STORAGE", 1 },
};
const ProtobufCEnumDescriptor spotify__download__proto__storage_resolve_response__result__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "spotify.download.proto.StorageResolveResponse.Result",
  "Result",
  "Spotify__Download__Proto__StorageResolveResponse__Result",
  "spotify.download.proto",
  3,
  spotify__download__proto__storage_resolve_response__result__enum_values_by_number,
  3,
  spotify__download__proto__storage_resolve_response__result__enum_values_by_name,
  2,
  spotify__download__proto__storage_resolve_response__result__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCFieldDescriptor spotify__download__proto__storage_resolve_response__field_descriptors[3] =
{
  {
    "result",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(Spotify__Download__Proto__StorageResolveResponse, result),
    &spotify__download__proto__storage_resolve_response__result__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cdnurl",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_STRING,
    offsetof(Spotify__Download__Proto__StorageResolveResponse, n_cdnurl),
    offsetof(Spotify__Download__Proto__StorageResolveResponse, cdnurl),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "fileid",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BYTES,
    0,   /* quantifier_offset */
    offsetof(Spotify__Download__Proto__StorageResolveResponse, fileid),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__download__proto__storage_resolve_response__field_indices_by_name[] = {
  1,   /* field[1] = cdnurl */
  2,   /* field[2] = fileid */
  0,   /* field[0] = result */
};
static const ProtobufCIntRange spotify__download__proto__storage_resolve_response__number_ranges[2 + 1] =
{
  { 1, 0 },
  { 4, 2 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__download__proto__storage_resolve_response__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.download.proto.StorageResolveResponse",
  "StorageResolveResponse",
  "Spotify__Download__Proto__StorageResolveResponse",
  "spotify.download.proto",
  sizeof(Spotify__Download__Proto__StorageResolveResponse),
  3,
  spotify__download__proto__storage_resolve_response__field_descriptors,
  spotify__download__proto__storage_resolve_response__field_indices_by_name,
  2,  spotify__download__proto__storage_resolve_response__number_ranges,
  (ProtobufCMessageInit) spotify__download__proto__storage_resolve_response__init,
  NULL,NULL,NULL    /* reserved[123] */
};
