/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: login5_credentials.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "login5_credentials.pb-c.h"
void   spotify__login5__v3__credentials__stored_credential__init
                     (Spotify__Login5__V3__Credentials__StoredCredential         *message)
{
  static const Spotify__Login5__V3__Credentials__StoredCredential init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__STORED_CREDENTIAL__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__stored_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__StoredCredential *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__stored_credential__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__stored_credential__pack
                     (const Spotify__Login5__V3__Credentials__StoredCredential *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__stored_credential__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__stored_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__StoredCredential *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__stored_credential__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__StoredCredential *
       spotify__login5__v3__credentials__stored_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__StoredCredential *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__stored_credential__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__stored_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__StoredCredential *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__stored_credential__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__password__init
                     (Spotify__Login5__V3__Credentials__Password         *message)
{
  static const Spotify__Login5__V3__Credentials__Password init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__PASSWORD__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__password__get_packed_size
                     (const Spotify__Login5__V3__Credentials__Password *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__password__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__password__pack
                     (const Spotify__Login5__V3__Credentials__Password *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__password__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__password__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__Password *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__password__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__Password *
       spotify__login5__v3__credentials__password__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__Password *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__password__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__password__free_unpacked
                     (Spotify__Login5__V3__Credentials__Password *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__password__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__facebook_access_token__init
                     (Spotify__Login5__V3__Credentials__FacebookAccessToken         *message)
{
  static const Spotify__Login5__V3__Credentials__FacebookAccessToken init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__FACEBOOK_ACCESS_TOKEN__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__facebook_access_token__get_packed_size
                     (const Spotify__Login5__V3__Credentials__FacebookAccessToken *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__facebook_access_token__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__facebook_access_token__pack
                     (const Spotify__Login5__V3__Credentials__FacebookAccessToken *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__facebook_access_token__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__facebook_access_token__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__FacebookAccessToken *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__facebook_access_token__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__FacebookAccessToken *
       spotify__login5__v3__credentials__facebook_access_token__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__FacebookAccessToken *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__facebook_access_token__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__facebook_access_token__free_unpacked
                     (Spotify__Login5__V3__Credentials__FacebookAccessToken *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__facebook_access_token__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__one_time_token__init
                     (Spotify__Login5__V3__Credentials__OneTimeToken         *message)
{
  static const Spotify__Login5__V3__Credentials__OneTimeToken init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__ONE_TIME_TOKEN__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__one_time_token__get_packed_size
                     (const Spotify__Login5__V3__Credentials__OneTimeToken *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__one_time_token__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__one_time_token__pack
                     (const Spotify__Login5__V3__Credentials__OneTimeToken *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__one_time_token__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__one_time_token__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__OneTimeToken *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__one_time_token__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__OneTimeToken *
       spotify__login5__v3__credentials__one_time_token__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__OneTimeToken *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__one_time_token__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__one_time_token__free_unpacked
                     (Spotify__Login5__V3__Credentials__OneTimeToken *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__one_time_token__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__parent_child_credential__init
                     (Spotify__Login5__V3__Credentials__ParentChildCredential         *message)
{
  static const Spotify__Login5__V3__Credentials__ParentChildCredential init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__PARENT_CHILD_CREDENTIAL__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__parent_child_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__ParentChildCredential *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__parent_child_credential__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__parent_child_credential__pack
                     (const Spotify__Login5__V3__Credentials__ParentChildCredential *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__parent_child_credential__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__parent_child_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__ParentChildCredential *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__parent_child_credential__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__ParentChildCredential *
       spotify__login5__v3__credentials__parent_child_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__ParentChildCredential *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__parent_child_credential__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__parent_child_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__ParentChildCredential *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__parent_child_credential__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__apple_sign_in_credential__init
                     (Spotify__Login5__V3__Credentials__AppleSignInCredential         *message)
{
  static const Spotify__Login5__V3__Credentials__AppleSignInCredential init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__APPLE_SIGN_IN_CREDENTIAL__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__apple_sign_in_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__AppleSignInCredential *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__apple_sign_in_credential__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__apple_sign_in_credential__pack
                     (const Spotify__Login5__V3__Credentials__AppleSignInCredential *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__apple_sign_in_credential__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__apple_sign_in_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__AppleSignInCredential *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__apple_sign_in_credential__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__AppleSignInCredential *
       spotify__login5__v3__credentials__apple_sign_in_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__AppleSignInCredential *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__apple_sign_in_credential__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__apple_sign_in_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__AppleSignInCredential *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__apple_sign_in_credential__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__samsung_sign_in_credential__init
                     (Spotify__Login5__V3__Credentials__SamsungSignInCredential         *message)
{
  static const Spotify__Login5__V3__Credentials__SamsungSignInCredential init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__SAMSUNG_SIGN_IN_CREDENTIAL__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__samsung_sign_in_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__SamsungSignInCredential *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__samsung_sign_in_credential__pack
                     (const Spotify__Login5__V3__Credentials__SamsungSignInCredential *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__samsung_sign_in_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__SamsungSignInCredential *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__SamsungSignInCredential *
       spotify__login5__v3__credentials__samsung_sign_in_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__SamsungSignInCredential *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__samsung_sign_in_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__SamsungSignInCredential *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__credentials__google_sign_in_credential__init
                     (Spotify__Login5__V3__Credentials__GoogleSignInCredential         *message)
{
  static const Spotify__Login5__V3__Credentials__GoogleSignInCredential init_value = SPOTIFY__LOGIN5__V3__CREDENTIALS__GOOGLE_SIGN_IN_CREDENTIAL__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__credentials__google_sign_in_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__GoogleSignInCredential *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__google_sign_in_credential__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__credentials__google_sign_in_credential__pack
                     (const Spotify__Login5__V3__Credentials__GoogleSignInCredential *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__google_sign_in_credential__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__credentials__google_sign_in_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__GoogleSignInCredential *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__credentials__google_sign_in_credential__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Credentials__GoogleSignInCredential *
       spotify__login5__v3__credentials__google_sign_in_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Credentials__GoogleSignInCredential *)
     protobuf_c_message_unpack (&spotify__login5__v3__credentials__google_sign_in_credential__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__credentials__google_sign_in_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__GoogleSignInCredential *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__credentials__google_sign_in_credential__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__stored_credential__field_descriptors[2] =
{
  {
    "username",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__StoredCredential, username),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "data",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BYTES,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__StoredCredential, data),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__stored_credential__field_indices_by_name[] = {
  1,   /* field[1] = data */
  0,   /* field[0] = username */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__stored_credential__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__stored_credential__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.StoredCredential",
  "StoredCredential",
  "Spotify__Login5__V3__Credentials__StoredCredential",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__StoredCredential),
  2,
  spotify__login5__v3__credentials__stored_credential__field_descriptors,
  spotify__login5__v3__credentials__stored_credential__field_indices_by_name,
  1,  spotify__login5__v3__credentials__stored_credential__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__stored_credential__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__password__field_descriptors[3] =
{
  {
    "id",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__Password, id),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "password",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__Password, password),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "padding",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BYTES,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__Password, padding),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__password__field_indices_by_name[] = {
  0,   /* field[0] = id */
  2,   /* field[2] = padding */
  1,   /* field[1] = password */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__password__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__password__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.Password",
  "Password",
  "Spotify__Login5__V3__Credentials__Password",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__Password),
  3,
  spotify__login5__v3__credentials__password__field_descriptors,
  spotify__login5__v3__credentials__password__field_indices_by_name,
  1,  spotify__login5__v3__credentials__password__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__password__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__facebook_access_token__field_descriptors[2] =
{
  {
    "fb_uid",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__FacebookAccessToken, fb_uid),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "access_token",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__FacebookAccessToken, access_token),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__facebook_access_token__field_indices_by_name[] = {
  1,   /* field[1] = access_token */
  0,   /* field[0] = fb_uid */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__facebook_access_token__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__facebook_access_token__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.FacebookAccessToken",
  "FacebookAccessToken",
  "Spotify__Login5__V3__Credentials__FacebookAccessToken",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__FacebookAccessToken),
  2,
  spotify__login5__v3__credentials__facebook_access_token__field_descriptors,
  spotify__login5__v3__credentials__facebook_access_token__field_indices_by_name,
  1,  spotify__login5__v3__credentials__facebook_access_token__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__facebook_access_token__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__one_time_token__field_descriptors[1] =
{
  {
    "token",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__OneTimeToken, token),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__one_time_token__field_indices_by_name[] = {
  0,   /* field[0] = token */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__one_time_token__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__one_time_token__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.OneTimeToken",
  "OneTimeToken",
  "Spotify__Login5__V3__Credentials__OneTimeToken",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__OneTimeToken),
  1,
  spotify__login5__v3__credentials__one_time_token__field_descriptors,
  spotify__login5__v3__credentials__one_time_token__field_indices_by_name,
  1,  spotify__login5__v3__credentials__one_time_token__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__one_time_token__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__parent_child_credential__field_descriptors[2] =
{
  {
    "child_id",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__ParentChildCredential, child_id),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "parent_stored_credential",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__ParentChildCredential, parent_stored_credential),
    &spotify__login5__v3__credentials__stored_credential__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__parent_child_credential__field_indices_by_name[] = {
  0,   /* field[0] = child_id */
  1,   /* field[1] = parent_stored_credential */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__parent_child_credential__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__parent_child_credential__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.ParentChildCredential",
  "ParentChildCredential",
  "Spotify__Login5__V3__Credentials__ParentChildCredential",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__ParentChildCredential),
  2,
  spotify__login5__v3__credentials__parent_child_credential__field_descriptors,
  spotify__login5__v3__credentials__parent_child_credential__field_indices_by_name,
  1,  spotify__login5__v3__credentials__parent_child_credential__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__parent_child_credential__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__apple_sign_in_credential__field_descriptors[3] =
{
  {
    "auth_code",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__AppleSignInCredential, auth_code),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "redirect_uri",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__AppleSignInCredential, redirect_uri),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "bundle_id",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__AppleSignInCredential, bundle_id),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__apple_sign_in_credential__field_indices_by_name[] = {
  0,   /* field[0] = auth_code */
  2,   /* field[2] = bundle_id */
  1,   /* field[1] = redirect_uri */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__apple_sign_in_credential__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__apple_sign_in_credential__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.AppleSignInCredential",
  "AppleSignInCredential",
  "Spotify__Login5__V3__Credentials__AppleSignInCredential",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__AppleSignInCredential),
  3,
  spotify__login5__v3__credentials__apple_sign_in_credential__field_descriptors,
  spotify__login5__v3__credentials__apple_sign_in_credential__field_indices_by_name,
  1,  spotify__login5__v3__credentials__apple_sign_in_credential__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__apple_sign_in_credential__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__samsung_sign_in_credential__field_descriptors[4] =
{
  {
    "auth_code",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__SamsungSignInCredential, auth_code),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "redirect_uri",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__SamsungSignInCredential, redirect_uri),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "id_token",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__SamsungSignInCredential, id_token),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "token_endpoint_url",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__SamsungSignInCredential, token_endpoint_url),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__samsung_sign_in_credential__field_indices_by_name[] = {
  0,   /* field[0] = auth_code */
  2,   /* field[2] = id_token */
  1,   /* field[1] = redirect_uri */
  3,   /* field[3] = token_endpoint_url */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__samsung_sign_in_credential__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.SamsungSignInCredential",
  "SamsungSignInCredential",
  "Spotify__Login5__V3__Credentials__SamsungSignInCredential",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__SamsungSignInCredential),
  4,
  spotify__login5__v3__credentials__samsung_sign_in_credential__field_descriptors,
  spotify__login5__v3__credentials__samsung_sign_in_credential__field_indices_by_name,
  1,  spotify__login5__v3__credentials__samsung_sign_in_credential__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__samsung_sign_in_credential__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__credentials__google_sign_in_credential__field_descriptors[2] =
{
  {
    "auth_code",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__GoogleSignInCredential, auth_code),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "redirect_uri",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Credentials__GoogleSignInCredential, redirect_uri),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__credentials__google_sign_in_credential__field_indices_by_name[] = {
  0,   /* field[0] = auth_code */
  1,   /* field[1] = redirect_uri */
};
static const ProtobufCIntRange spotify__login5__v3__credentials__google_sign_in_credential__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__credentials__google_sign_in_credential__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.credentials.GoogleSignInCredential",
  "GoogleSignInCredential",
  "Spotify__Login5__V3__Credentials__GoogleSignInCredential",
  "spotify.login5.v3.credentials",
  sizeof(Spotify__Login5__V3__Credentials__GoogleSignInCredential),
  2,
  spotify__login5__v3__credentials__google_sign_in_credential__field_descriptors,
  spotify__login5__v3__credentials__google_sign_in_credential__field_indices_by_name,
  1,  spotify__login5__v3__credentials__google_sign_in_credential__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__credentials__google_sign_in_credential__init,
  NULL,NULL,NULL    /* reserved[123] */
};
