/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: extended_metadata.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "extended_metadata.pb-c.h"
void   spotify__extendedmetadata__extension_query__init
                     (Spotify__Extendedmetadata__ExtensionQuery         *message)
{
  static const Spotify__Extendedmetadata__ExtensionQuery init_value = SPOTIFY__EXTENDEDMETADATA__EXTENSION_QUERY__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__extension_query__get_packed_size
                     (const Spotify__Extendedmetadata__ExtensionQuery *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__extension_query__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__extension_query__pack
                     (const Spotify__Extendedmetadata__ExtensionQuery *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__extension_query__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__extension_query__pack_to_buffer
                     (const Spotify__Extendedmetadata__ExtensionQuery *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__extension_query__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__ExtensionQuery *
       spotify__extendedmetadata__extension_query__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__ExtensionQuery *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__extension_query__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__extension_query__free_unpacked
                     (Spotify__Extendedmetadata__ExtensionQuery *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__extension_query__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__entity_request__init
                     (Spotify__Extendedmetadata__EntityRequest         *message)
{
  static const Spotify__Extendedmetadata__EntityRequest init_value = SPOTIFY__EXTENDEDMETADATA__ENTITY_REQUEST__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__entity_request__get_packed_size
                     (const Spotify__Extendedmetadata__EntityRequest *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_request__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__entity_request__pack
                     (const Spotify__Extendedmetadata__EntityRequest *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_request__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__entity_request__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityRequest *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_request__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__EntityRequest *
       spotify__extendedmetadata__entity_request__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__EntityRequest *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__entity_request__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__entity_request__free_unpacked
                     (Spotify__Extendedmetadata__EntityRequest *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_request__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__batched_entity_request_header__init
                     (Spotify__Extendedmetadata__BatchedEntityRequestHeader         *message)
{
  static const Spotify__Extendedmetadata__BatchedEntityRequestHeader init_value = SPOTIFY__EXTENDEDMETADATA__BATCHED_ENTITY_REQUEST_HEADER__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__batched_entity_request_header__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedEntityRequestHeader *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request_header__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__batched_entity_request_header__pack
                     (const Spotify__Extendedmetadata__BatchedEntityRequestHeader *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request_header__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__batched_entity_request_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedEntityRequestHeader *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request_header__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__BatchedEntityRequestHeader *
       spotify__extendedmetadata__batched_entity_request_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__BatchedEntityRequestHeader *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__batched_entity_request_header__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__batched_entity_request_header__free_unpacked
                     (Spotify__Extendedmetadata__BatchedEntityRequestHeader *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request_header__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__batched_entity_request__init
                     (Spotify__Extendedmetadata__BatchedEntityRequest         *message)
{
  static const Spotify__Extendedmetadata__BatchedEntityRequest init_value = SPOTIFY__EXTENDEDMETADATA__BATCHED_ENTITY_REQUEST__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__batched_entity_request__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedEntityRequest *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__batched_entity_request__pack
                     (const Spotify__Extendedmetadata__BatchedEntityRequest *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__batched_entity_request__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedEntityRequest *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__BatchedEntityRequest *
       spotify__extendedmetadata__batched_entity_request__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__BatchedEntityRequest *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__batched_entity_request__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__batched_entity_request__free_unpacked
                     (Spotify__Extendedmetadata__BatchedEntityRequest *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_entity_request__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__entity_extension_data_array_header__init
                     (Spotify__Extendedmetadata__EntityExtensionDataArrayHeader         *message)
{
  static const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader init_value = SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA_ARRAY_HEADER__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__entity_extension_data_array_header__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array_header__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__entity_extension_data_array_header__pack
                     (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array_header__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__entity_extension_data_array_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array_header__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *
       spotify__extendedmetadata__entity_extension_data_array_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__entity_extension_data_array_header__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__entity_extension_data_array_header__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array_header__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__entity_extension_data_array__init
                     (Spotify__Extendedmetadata__EntityExtensionDataArray         *message)
{
  static const Spotify__Extendedmetadata__EntityExtensionDataArray init_value = SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA_ARRAY__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__entity_extension_data_array__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionDataArray *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__entity_extension_data_array__pack
                     (const Spotify__Extendedmetadata__EntityExtensionDataArray *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__entity_extension_data_array__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionDataArray *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__EntityExtensionDataArray *
       spotify__extendedmetadata__entity_extension_data_array__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__EntityExtensionDataArray *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__entity_extension_data_array__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__entity_extension_data_array__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionDataArray *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_array__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__batched_extension_response_header__init
                     (Spotify__Extendedmetadata__BatchedExtensionResponseHeader         *message)
{
  static const Spotify__Extendedmetadata__BatchedExtensionResponseHeader init_value = SPOTIFY__EXTENDEDMETADATA__BATCHED_EXTENSION_RESPONSE_HEADER__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__batched_extension_response_header__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response_header__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__batched_extension_response_header__pack
                     (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response_header__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__batched_extension_response_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response_header__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__BatchedExtensionResponseHeader *
       spotify__extendedmetadata__batched_extension_response_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__BatchedExtensionResponseHeader *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__batched_extension_response_header__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__batched_extension_response_header__free_unpacked
                     (Spotify__Extendedmetadata__BatchedExtensionResponseHeader *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response_header__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__batched_extension_response__init
                     (Spotify__Extendedmetadata__BatchedExtensionResponse         *message)
{
  static const Spotify__Extendedmetadata__BatchedExtensionResponse init_value = SPOTIFY__EXTENDEDMETADATA__BATCHED_EXTENSION_RESPONSE__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__batched_extension_response__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedExtensionResponse *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__batched_extension_response__pack
                     (const Spotify__Extendedmetadata__BatchedExtensionResponse *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__batched_extension_response__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedExtensionResponse *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__BatchedExtensionResponse *
       spotify__extendedmetadata__batched_extension_response__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__BatchedExtensionResponse *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__batched_extension_response__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__batched_extension_response__free_unpacked
                     (Spotify__Extendedmetadata__BatchedExtensionResponse *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__batched_extension_response__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCFieldDescriptor spotify__extendedmetadata__extension_query__field_descriptors[2] =
{
  {
    "extension_kind",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__ExtensionQuery, extension_kind),
    &spotify__extendedmetadata__extension_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "etag",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__ExtensionQuery, etag),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__extension_query__field_indices_by_name[] = {
  1,   /* field[1] = etag */
  0,   /* field[0] = extension_kind */
};
static const ProtobufCIntRange spotify__extendedmetadata__extension_query__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__extension_query__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.ExtensionQuery",
  "ExtensionQuery",
  "Spotify__Extendedmetadata__ExtensionQuery",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__ExtensionQuery),
  2,
  spotify__extendedmetadata__extension_query__field_descriptors,
  spotify__extendedmetadata__extension_query__field_indices_by_name,
  1,  spotify__extendedmetadata__extension_query__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__extension_query__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__entity_request__field_descriptors[2] =
{
  {
    "entity_uri",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityRequest, entity_uri),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "query",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Extendedmetadata__EntityRequest, n_query),
    offsetof(Spotify__Extendedmetadata__EntityRequest, query),
    &spotify__extendedmetadata__extension_query__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__entity_request__field_indices_by_name[] = {
  0,   /* field[0] = entity_uri */
  1,   /* field[1] = query */
};
static const ProtobufCIntRange spotify__extendedmetadata__entity_request__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_request__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.EntityRequest",
  "EntityRequest",
  "Spotify__Extendedmetadata__EntityRequest",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__EntityRequest),
  2,
  spotify__extendedmetadata__entity_request__field_descriptors,
  spotify__extendedmetadata__entity_request__field_indices_by_name,
  1,  spotify__extendedmetadata__entity_request__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__entity_request__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__batched_entity_request_header__field_descriptors[3] =
{
  {
    "country",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__BatchedEntityRequestHeader, country),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "catalogue",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__BatchedEntityRequestHeader, catalogue),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "task_id",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BYTES,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__BatchedEntityRequestHeader, task_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__batched_entity_request_header__field_indices_by_name[] = {
  1,   /* field[1] = catalogue */
  0,   /* field[0] = country */
  2,   /* field[2] = task_id */
};
static const ProtobufCIntRange spotify__extendedmetadata__batched_entity_request_header__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_entity_request_header__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.BatchedEntityRequestHeader",
  "BatchedEntityRequestHeader",
  "Spotify__Extendedmetadata__BatchedEntityRequestHeader",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__BatchedEntityRequestHeader),
  3,
  spotify__extendedmetadata__batched_entity_request_header__field_descriptors,
  spotify__extendedmetadata__batched_entity_request_header__field_indices_by_name,
  1,  spotify__extendedmetadata__batched_entity_request_header__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__batched_entity_request_header__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__batched_entity_request__field_descriptors[2] =
{
  {
    "header",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__BatchedEntityRequest, header),
    &spotify__extendedmetadata__batched_entity_request_header__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "entity_request",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Extendedmetadata__BatchedEntityRequest, n_entity_request),
    offsetof(Spotify__Extendedmetadata__BatchedEntityRequest, entity_request),
    &spotify__extendedmetadata__entity_request__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__batched_entity_request__field_indices_by_name[] = {
  1,   /* field[1] = entity_request */
  0,   /* field[0] = header */
};
static const ProtobufCIntRange spotify__extendedmetadata__batched_entity_request__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_entity_request__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.BatchedEntityRequest",
  "BatchedEntityRequest",
  "Spotify__Extendedmetadata__BatchedEntityRequest",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__BatchedEntityRequest),
  2,
  spotify__extendedmetadata__batched_entity_request__field_descriptors,
  spotify__extendedmetadata__batched_entity_request__field_indices_by_name,
  1,  spotify__extendedmetadata__batched_entity_request__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__batched_entity_request__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__entity_extension_data_array_header__field_descriptors[4] =
{
  {
    "provider_error_status",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArrayHeader, provider_error_status),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cache_ttl_in_seconds",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT64,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArrayHeader, cache_ttl_in_seconds),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "offline_ttl_in_seconds",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT64,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArrayHeader, offline_ttl_in_seconds),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "extension_type",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArrayHeader, extension_type),
    &spotify__extendedmetadata__extension_type__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__entity_extension_data_array_header__field_indices_by_name[] = {
  1,   /* field[1] = cache_ttl_in_seconds */
  3,   /* field[3] = extension_type */
  2,   /* field[2] = offline_ttl_in_seconds */
  0,   /* field[0] = provider_error_status */
};
static const ProtobufCIntRange spotify__extendedmetadata__entity_extension_data_array_header__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data_array_header__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.EntityExtensionDataArrayHeader",
  "EntityExtensionDataArrayHeader",
  "Spotify__Extendedmetadata__EntityExtensionDataArrayHeader",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__EntityExtensionDataArrayHeader),
  4,
  spotify__extendedmetadata__entity_extension_data_array_header__field_descriptors,
  spotify__extendedmetadata__entity_extension_data_array_header__field_indices_by_name,
  1,  spotify__extendedmetadata__entity_extension_data_array_header__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__entity_extension_data_array_header__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__entity_extension_data_array__field_descriptors[3] =
{
  {
    "header",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArray, header),
    &spotify__extendedmetadata__entity_extension_data_array_header__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "extension_kind",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArray, extension_kind),
    &spotify__extendedmetadata__extension_kind__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "extension_data",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArray, n_extension_data),
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataArray, extension_data),
    &spotify__extendedmetadata__entity_extension_data__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__entity_extension_data_array__field_indices_by_name[] = {
  2,   /* field[2] = extension_data */
  1,   /* field[1] = extension_kind */
  0,   /* field[0] = header */
};
static const ProtobufCIntRange spotify__extendedmetadata__entity_extension_data_array__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data_array__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.EntityExtensionDataArray",
  "EntityExtensionDataArray",
  "Spotify__Extendedmetadata__EntityExtensionDataArray",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__EntityExtensionDataArray),
  3,
  spotify__extendedmetadata__entity_extension_data_array__field_descriptors,
  spotify__extendedmetadata__entity_extension_data_array__field_indices_by_name,
  1,  spotify__extendedmetadata__entity_extension_data_array__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__entity_extension_data_array__init,
  NULL,NULL,NULL    /* reserved[123] */
};
#define spotify__extendedmetadata__batched_extension_response_header__field_descriptors NULL
#define spotify__extendedmetadata__batched_extension_response_header__field_indices_by_name NULL
#define spotify__extendedmetadata__batched_extension_response_header__number_ranges NULL
const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_extension_response_header__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.BatchedExtensionResponseHeader",
  "BatchedExtensionResponseHeader",
  "Spotify__Extendedmetadata__BatchedExtensionResponseHeader",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__BatchedExtensionResponseHeader),
  0,
  spotify__extendedmetadata__batched_extension_response_header__field_descriptors,
  spotify__extendedmetadata__batched_extension_response_header__field_indices_by_name,
  0,  spotify__extendedmetadata__batched_extension_response_header__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__batched_extension_response_header__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__batched_extension_response__field_descriptors[2] =
{
  {
    "header",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__BatchedExtensionResponse, header),
    &spotify__extendedmetadata__batched_extension_response_header__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "extended_metadata",
    2,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Extendedmetadata__BatchedExtensionResponse, n_extended_metadata),
    offsetof(Spotify__Extendedmetadata__BatchedExtensionResponse, extended_metadata),
    &spotify__extendedmetadata__entity_extension_data_array__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__batched_extension_response__field_indices_by_name[] = {
  1,   /* field[1] = extended_metadata */
  0,   /* field[0] = header */
};
static const ProtobufCIntRange spotify__extendedmetadata__batched_extension_response__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_extension_response__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.BatchedExtensionResponse",
  "BatchedExtensionResponse",
  "Spotify__Extendedmetadata__BatchedExtensionResponse",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__BatchedExtensionResponse),
  2,
  spotify__extendedmetadata__batched_extension_response__field_descriptors,
  spotify__extendedmetadata__batched_extension_response__field_indices_by_name,
  1,  spotify__extendedmetadata__batched_extension_response__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__batched_extension_response__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCEnumValue spotify__extendedmetadata__extension_type__enum_values_by_number[3] =
{
  { "UNKNOWN", "SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__UNKNOWN", 0 },
  { "GENERIC", "SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__GENERIC", 1 },
  { "ASSOC", "SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__ASSOC", 2 },
};
static const ProtobufCIntRange spotify__extendedmetadata__extension_type__value_ranges[] = {
{0, 0},{0, 3}
};
static const ProtobufCEnumValueIndex spotify__extendedmetadata__extension_type__enum_values_by_name[3] =
{
  { "ASSOC", 2 },
  { "GENERIC", 1 },
  { "UNKNOWN", 0 },
};
const ProtobufCEnumDescriptor spotify__extendedmetadata__extension_type__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.ExtensionType",
  "ExtensionType",
  "Spotify__Extendedmetadata__ExtensionType",
  "spotify.extendedmetadata",
  3,
  spotify__extendedmetadata__extension_type__enum_values_by_number,
  3,
  spotify__extendedmetadata__extension_type__enum_values_by_name,
  1,
  spotify__extendedmetadata__extension_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
