/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: login5_challenges_hashcash.proto */

#ifndef PROTOBUF_C_login5_5fchallenges_5fhashcash_2eproto__INCLUDED
#define PROTOBUF_C_login5_5fchallenges_5fhashcash_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif

#include "google_duration.pb-c.h"

typedef struct Spotify__Login5__V3__Challenges__HashcashChallenge Spotify__Login5__V3__Challenges__HashcashChallenge;
typedef struct Spotify__Login5__V3__Challenges__HashcashSolution Spotify__Login5__V3__Challenges__HashcashSolution;


/* --- enums --- */


/* --- messages --- */

struct  Spotify__Login5__V3__Challenges__HashcashChallenge
{
  ProtobufCMessage base;
  ProtobufCBinaryData prefix;
  int32_t length;
};
#define SPOTIFY__LOGIN5__V3__CHALLENGES__HASHCASH_CHALLENGE__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__challenges__hashcash_challenge__descriptor) \
    , {0,NULL}, 0 }


struct  Spotify__Login5__V3__Challenges__HashcashSolution
{
  ProtobufCMessage base;
  ProtobufCBinaryData suffix;
  Google__Protobuf__Duration *duration;
};
#define SPOTIFY__LOGIN5__V3__CHALLENGES__HASHCASH_SOLUTION__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__challenges__hashcash_solution__descriptor) \
    , {0,NULL}, NULL }


/* Spotify__Login5__V3__Challenges__HashcashChallenge methods */
void   spotify__login5__v3__challenges__hashcash_challenge__init
                     (Spotify__Login5__V3__Challenges__HashcashChallenge         *message);
size_t spotify__login5__v3__challenges__hashcash_challenge__get_packed_size
                     (const Spotify__Login5__V3__Challenges__HashcashChallenge   *message);
size_t spotify__login5__v3__challenges__hashcash_challenge__pack
                     (const Spotify__Login5__V3__Challenges__HashcashChallenge   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__challenges__hashcash_challenge__pack_to_buffer
                     (const Spotify__Login5__V3__Challenges__HashcashChallenge   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Challenges__HashcashChallenge *
       spotify__login5__v3__challenges__hashcash_challenge__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__challenges__hashcash_challenge__free_unpacked
                     (Spotify__Login5__V3__Challenges__HashcashChallenge *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Challenges__HashcashSolution methods */
void   spotify__login5__v3__challenges__hashcash_solution__init
                     (Spotify__Login5__V3__Challenges__HashcashSolution         *message);
size_t spotify__login5__v3__challenges__hashcash_solution__get_packed_size
                     (const Spotify__Login5__V3__Challenges__HashcashSolution   *message);
size_t spotify__login5__v3__challenges__hashcash_solution__pack
                     (const Spotify__Login5__V3__Challenges__HashcashSolution   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__challenges__hashcash_solution__pack_to_buffer
                     (const Spotify__Login5__V3__Challenges__HashcashSolution   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Challenges__HashcashSolution *
       spotify__login5__v3__challenges__hashcash_solution__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__challenges__hashcash_solution__free_unpacked
                     (Spotify__Login5__V3__Challenges__HashcashSolution *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Login5__V3__Challenges__HashcashChallenge_Closure)
                 (const Spotify__Login5__V3__Challenges__HashcashChallenge *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Challenges__HashcashSolution_Closure)
                 (const Spotify__Login5__V3__Challenges__HashcashSolution *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__login5__v3__challenges__hashcash_challenge__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__challenges__hashcash_solution__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_login5_5fchallenges_5fhashcash_2eproto__INCLUDED */
