syntax = "proto3";

package spotify.extendedmetadata;

enum ExtensionKind {
  UNKNOWN_EXTENSION = 0;
  CANVAZ = 1;
  STORYLINES = 2;
  PODCAST_TOPICS = 3;
  PODCAST_SEGMENTS = 4;
  AUDIO_FILES = 5;
  TRACK_DESCRIPTOR = 6;
  PODCAST_COUNTER = 7;
  ARTIST_V4 = 8;
  ALBUM_V4 = 9;
  TRACK_V4 = 10;
  SHOW_V4 = 11;
  EPISODE_V4 = 12;
  PODCAST_HTML_DESCRIPTION = 13;
  PODCAST_QUOTES = 14;
  USER_PROFILE = 15;
  CANVAS_V1 = 16;
  SHOW_V4_BASE = 17;
  SHOW_V4_EPISODES_ASSOC = 18;
  TRACK_DESCRIPTOR_SIGNATURES = 19;
  PODCAST_AD_SEGMENTS = 20;
  EPISODE_TRANSCRIPTS = 21;
  PODCAST_SUBSCRIPTIONS = 22;
  EXTRACTED_COLOR = 23;
  PODCAST_VIRALITY = 24;
  IMAGE_SPARKLES_HACK = 25;
  PODCAST_POPULARITY_HACK = 26;
  AUTOMIX_MODE = 27;
  CUEPOINTS = 28;
  PODCAST_POLL = 29;
  EPISODE_ACCESS = 30;
  SHOW_ACCESS = 31;
  PODCAST_QNA = 32;
  CLIPS = 33;
  SHOW_V5 = 34;
  EPISODE_V5 = 35;
  PODCAST_CTA_CARDS = 36;
  PODCAST_RATING = 37;
  DISPLAY_SEGMENTS = 38;
  GREENROOM = 39;
  USER_CREATED = 40;
  SHOW_DESCRIPTION = 41;
  SHOW_HTML_DESCRIPTION = 42;
  SHOW_PLAYABILITY = 43;
  EPISODE_DESCRIPTION = 44;
  EPISODE_HTML_DESCRIPTION = 45;
  EPISODE_PLAYABILITY = 46;
  SHOW_EPISODES_ASSOC = 47;
  CLIENT_CONFIG = 48;
  PLAYLISTABILITY = 49;
  AUDIOBOOK_V5 = 50;
  CHAPTER_V5 = 51;
  AUDIOBOOK_SPECIFICS = 52;
  EPISODE_RANKING = 53;
  HTML_DESCRIPTION = 54;
  CREATOR_CHANNEL = 55;
  AUDIOBOOK_PROVIDERS = 56;
  PLAY_TRAIT = 57;
  CONTENT_WARNING = 58;
  IMAGE_CUE = 59;
  STREAM_COUNT = 60;
  AUDIO_ATTRIBUTES = 61;
  NAVIGABLE_TRAIT = 62;
  NEXT_BEST_EPISODE = 63;
  AUDIOBOOK_PRICE = 64;
  EXPRESSIVE_PLAYLISTS = 65;
  DYNAMIC_SHOW_EPISODE = 66;
  LIVE = 67;
  SKIP_PLAYED = 68;
  AD_BREAK_FREE_PODCASTS = 69;
  ASSOCIATIONS = 70;
  PLAYLIST_EVALUATION = 71;
  CACHE_INVALIDATIONS = 72;
  LIVESTREAM_ENTITY = 73;
  SINGLE_TAP_REACTIONS = 74;
  USER_COMMENTS = 75;
  CLIENT_RESTRICTIONS = 76;
  PODCAST_GUEST = 77;
  PLAYABILITY = 78;
  COVER_IMAGE = 79;
  SHARE_TRAIT = 80;
  INSTANCE_SHARING = 81;
  ARTIST_TOUR = 82;
  AUDIOBOOK_GENRE = 83;
  CONCEPT = 84;
  ORIGINAL_VIDEO = 85;
  SMART_SHUFFLE = 86;
  LIVE_EVENTS = 87;
  AUDIOBOOK_RELATIONS = 88;
  HOME_POC_BASECARD = 89;
  AUDIOBOOK_SUPPLEMENTS = 90;
  PAID_PODCAST_BANNER = 91;
  FEWER_ADS = 92;
  WATCH_FEED_SHOW_EXPLORER = 93;
  TRACK_EXTRA_DESCRIPTORS = 94;
  TRACK_EXTRA_AUDIO_ATTRIBUTES = 95;
  TRACK_EXTENDED_CREDITS = 96;
  SIMPLE_TRAIT = 97;
  AUDIO_ASSOCIATIONS = 98;
  VIDEO_ASSOCIATIONS = 99;
  PLAYLIST_TUNER = 100;
  ARTIST_VIDEOS_ENTRYPOINT = 101;
  ALBUM_PRERELEASE = 102;
  CONTENT_ALTERNATIVES = 103;
  SNAPSHOT_SHARING = 105;
  DISPLAY_SEGMENTS_COUNT = 106;
  PODCAST_FEATURED_EPISODE = 107;
  PODCAST_SPONSORED_CONTENT = 108;
  PODCAST_EPISODE_TOPICS_LLM = 109;
  PODCAST_EPISODE_TOPICS_KG = 110;
  EPISODE_RANKING_POPULARITY = 111;
  MERCH = 112;
  COMPANION_CONTENT = 113;
  WATCH_FEED_ENTITY_EXPLORER = 114;
  ANCHOR_CARD_TRAIT = 115;
  AUDIO_PREVIEW_PLAYBACK_TRAIT = 116;
  VIDEO_PREVIEW_STILL_TRAIT = 117;
  PREVIEW_CARD_TRAIT = 118;
  SHORTCUTS_CARD_TRAIT = 119;
  VIDEO_PREVIEW_PLAYBACK_TRAIT = 120;
  COURSE_SPECIFICS = 121;
  CONCERT = 122;
  CONCERT_LOCATION = 123;
  CONCERT_MARKETING = 124;
  CONCERT_PERFORMERS = 125;
  TRACK_PAIR_TRANSITION = 126;
  CONTENT_TYPE_TRAIT = 127;
  NAME_TRAIT = 128;
  ARTWORK_TRAIT = 129;
  RELEASE_DATE_TRAIT = 130;
  CREDITS_TRAIT = 131;
  RELEASE_URI_TRAIT = 132;
  ENTITY_CAPPING = 133;
  LESSON_SPECIFICS = 134;
  CONCERT_OFFERS = 135;
  TRANSITION_MAPS = 136;
  ARTIST_HAS_CONCERTS = 137;
  PRERELEASE = 138;
  PLAYLIST_ATTRIBUTES_V2 = 139;
  LIST_ATTRIBUTES_V2 = 140;
  LIST_METADATA = 141;
  LIST_TUNER_AUDIO_ANALYSIS = 142;
  LIST_TUNER_CUEPOINTS = 143;
  CONTENT_RATING_TRAIT = 144;
  COPYRIGHT_TRAIT = 145;
  SUPPORTED_BADGES = 146;
  BADGES = 147;
  PREVIEW_TRAIT = 148;
  ROOTLISTABILITY_TRAIT = 149;
  LOCAL_CONCERTS = 150;
  RECOMMENDED_PLAYLISTS = 151;
  POPULAR_RELEASES = 152;
  RELATED_RELEASES = 153;
  SHARE_RESTRICTIONS = 154;
  CONCERT_OFFER = 155;
  CONCERT_OFFER_PROVIDER = 156;
  ENTITY_BOOKMARKS = 157;
  PRIVACY_TRAIT = 158;
  DUPLICATE_ITEMS_TRAIT = 159;
  REORDERING_TRAIT = 160;
  PODCAST_RESUMPTION_SEGMENTS = 161;
  ARTIST_EXPRESSION_VIDEO = 162;
  PRERELEASE_VIDEO = 163;
  GATED_ENTITY_RELATIONS = 164;
  RELATED_CREATORS_SECTION = 165;
  CREATORS_APPEARS_ON_SECTION = 166;
  PROMO_V1_TRAIT = 167;
  SPEECHLESS_SHARE_CARD = 168;
  TOP_PLAYABLES_SECTION = 169;
  AUTO_LENS = 170;
  PROMO_V3_TRAIT = 171;
  TRACK_CONTENT_FILTER = 172;
  HIGHLIGHTABILITY = 173;
  LINK_CARD_WITH_IMAGE_TRAIT = 174;
  TRACK_CLOUD_SECTION = 175;
  EPISODE_TOPICS = 176;
  VIDEO_THUMBNAIL = 177;
  IDENTITY_TRAIT = 178;
  VISUAL_IDENTITY_TRAIT = 179;
  CONTENT_TYPE_V2_TRAIT = 180;
  PREVIEW_PLAYBACK_TRAIT = 181;
  CONSUMPTION_EXPERIENCE_TRAIT = 182;
  PUBLISHING_METADATA_TRAIT = 183;
  DETAILED_EVALUATION_TRAIT = 184;
  ON_PLATFORM_REPUTATION_TRAIT = 185;
  CREDITS_V2_TRAIT = 186;
  HIGHLIGHT_PLAYABILITY_TRAIT = 187;
  SHOW_EPISODE_LIST = 188;
  AVAILABLE_RELEASES = 189;
}
