/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: connectivity.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "connectivity.pb-c.h"
void   spotify__clienttoken__data__v0__connectivity_sdk_data__init
                     (Spotify__Clienttoken__Data__V0__ConnectivitySdkData         *message)
{
  static const Spotify__Clienttoken__Data__V0__ConnectivitySdkData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__CONNECTIVITY_SDK_DATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__connectivity_sdk_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__connectivity_sdk_data__pack
                     (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__connectivity_sdk_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__ConnectivitySdkData *
       spotify__clienttoken__data__v0__connectivity_sdk_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__ConnectivitySdkData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__connectivity_sdk_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__ConnectivitySdkData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__platform_specific_data__init
                     (Spotify__Clienttoken__Data__V0__PlatformSpecificData         *message)
{
  static const Spotify__Clienttoken__Data__V0__PlatformSpecificData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__platform_specific_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__PlatformSpecificData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__platform_specific_data__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__platform_specific_data__pack
                     (const Spotify__Clienttoken__Data__V0__PlatformSpecificData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__platform_specific_data__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__platform_specific_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__PlatformSpecificData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__platform_specific_data__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__PlatformSpecificData *
       spotify__clienttoken__data__v0__platform_specific_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__PlatformSpecificData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__platform_specific_data__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__platform_specific_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__PlatformSpecificData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__platform_specific_data__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__native_android_data__init
                     (Spotify__Clienttoken__Data__V0__NativeAndroidData         *message)
{
  static const Spotify__Clienttoken__Data__V0__NativeAndroidData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_ANDROID_DATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__native_android_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeAndroidData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_android_data__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__native_android_data__pack
                     (const Spotify__Clienttoken__Data__V0__NativeAndroidData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_android_data__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__native_android_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeAndroidData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_android_data__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__NativeAndroidData *
       spotify__clienttoken__data__v0__native_android_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__NativeAndroidData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__native_android_data__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__native_android_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeAndroidData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_android_data__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__native_iosdata__init
                     (Spotify__Clienttoken__Data__V0__NativeIOSData         *message)
{
  static const Spotify__Clienttoken__Data__V0__NativeIOSData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_IOSDATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__native_iosdata__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeIOSData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_iosdata__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__native_iosdata__pack
                     (const Spotify__Clienttoken__Data__V0__NativeIOSData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_iosdata__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__native_iosdata__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeIOSData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_iosdata__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__NativeIOSData *
       spotify__clienttoken__data__v0__native_iosdata__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__NativeIOSData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__native_iosdata__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__native_iosdata__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeIOSData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_iosdata__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__native_desktop_windows_data__init
                     (Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData         *message)
{
  static const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_DESKTOP_WINDOWS_DATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__native_desktop_windows_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__native_desktop_windows_data__pack
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__native_desktop_windows_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *
       spotify__clienttoken__data__v0__native_desktop_windows_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__native_desktop_windows_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__native_desktop_linux_data__init
                     (Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData         *message)
{
  static const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_DESKTOP_LINUX_DATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__native_desktop_linux_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__native_desktop_linux_data__pack
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__native_desktop_linux_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *
       spotify__clienttoken__data__v0__native_desktop_linux_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__native_desktop_linux_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__native_desktop_mac_osdata__init
                     (Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData         *message)
{
  static const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_DESKTOP_MAC_OSDATA__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__native_desktop_mac_osdata__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__native_desktop_mac_osdata__pack
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__native_desktop_mac_osdata__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *
       spotify__clienttoken__data__v0__native_desktop_mac_osdata__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__native_desktop_mac_osdata__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__clienttoken__data__v0__screen__init
                     (Spotify__Clienttoken__Data__V0__Screen         *message)
{
  static const Spotify__Clienttoken__Data__V0__Screen init_value = SPOTIFY__CLIENTTOKEN__DATA__V0__SCREEN__INIT;
  *message = init_value;
}
size_t spotify__clienttoken__data__v0__screen__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__Screen *message)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__screen__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__clienttoken__data__v0__screen__pack
                     (const Spotify__Clienttoken__Data__V0__Screen *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__screen__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__clienttoken__data__v0__screen__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__Screen *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__screen__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Clienttoken__Data__V0__Screen *
       spotify__clienttoken__data__v0__screen__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Clienttoken__Data__V0__Screen *)
     protobuf_c_message_unpack (&spotify__clienttoken__data__v0__screen__descriptor,
                                allocator, len, data);
}
void   spotify__clienttoken__data__v0__screen__free_unpacked
                     (Spotify__Clienttoken__Data__V0__Screen *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__clienttoken__data__v0__screen__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__connectivity_sdk_data__field_descriptors[2] =
{
  {
    "platform_specific_data",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__ConnectivitySdkData, platform_specific_data),
    &spotify__clienttoken__data__v0__platform_specific_data__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "device_id",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__ConnectivitySdkData, device_id),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__connectivity_sdk_data__field_indices_by_name[] = {
  1,   /* field[1] = device_id */
  0,   /* field[0] = platform_specific_data */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__connectivity_sdk_data__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.ConnectivitySdkData",
  "ConnectivitySdkData",
  "Spotify__Clienttoken__Data__V0__ConnectivitySdkData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__ConnectivitySdkData),
  2,
  spotify__clienttoken__data__v0__connectivity_sdk_data__field_descriptors,
  spotify__clienttoken__data__v0__connectivity_sdk_data__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__connectivity_sdk_data__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__connectivity_sdk_data__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__platform_specific_data__field_descriptors[5] =
{
  {
    "android",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, data_case),
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, android),
    &spotify__clienttoken__data__v0__native_android_data__descriptor,
    NULL,
    0 | PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "ios",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, data_case),
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, ios),
    &spotify__clienttoken__data__v0__native_iosdata__descriptor,
    NULL,
    0 | PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "desktop_macos",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, data_case),
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, desktop_macos),
    &spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor,
    NULL,
    0 | PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "desktop_windows",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, data_case),
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, desktop_windows),
    &spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor,
    NULL,
    0 | PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "desktop_linux",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, data_case),
    offsetof(Spotify__Clienttoken__Data__V0__PlatformSpecificData, desktop_linux),
    &spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor,
    NULL,
    0 | PROTOBUF_C_FIELD_FLAG_ONEOF,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__platform_specific_data__field_indices_by_name[] = {
  0,   /* field[0] = android */
  4,   /* field[4] = desktop_linux */
  2,   /* field[2] = desktop_macos */
  3,   /* field[3] = desktop_windows */
  1,   /* field[1] = ios */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__platform_specific_data__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__platform_specific_data__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.PlatformSpecificData",
  "PlatformSpecificData",
  "Spotify__Clienttoken__Data__V0__PlatformSpecificData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__PlatformSpecificData),
  5,
  spotify__clienttoken__data__v0__platform_specific_data__field_descriptors,
  spotify__clienttoken__data__v0__platform_specific_data__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__platform_specific_data__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__platform_specific_data__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__native_android_data__field_descriptors[8] =
{
  {
    "screen_dimensions",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, screen_dimensions),
    &spotify__clienttoken__data__v0__screen__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "android_version",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, android_version),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "api_version",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, api_version),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "device_name",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, device_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "model_str",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, model_str),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vendor",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, vendor),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "vendor_2",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, vendor_2),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_value_8",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeAndroidData, unknown_value_8),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__native_android_data__field_indices_by_name[] = {
  1,   /* field[1] = android_version */
  2,   /* field[2] = api_version */
  3,   /* field[3] = device_name */
  4,   /* field[4] = model_str */
  0,   /* field[0] = screen_dimensions */
  7,   /* field[7] = unknown_value_8 */
  5,   /* field[5] = vendor */
  6,   /* field[6] = vendor_2 */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__native_android_data__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_android_data__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.NativeAndroidData",
  "NativeAndroidData",
  "Spotify__Clienttoken__Data__V0__NativeAndroidData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__NativeAndroidData),
  8,
  spotify__clienttoken__data__v0__native_android_data__field_descriptors,
  spotify__clienttoken__data__v0__native_android_data__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__native_android_data__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__native_android_data__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__native_iosdata__field_descriptors[5] =
{
  {
    "user_interface_idiom",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeIOSData, user_interface_idiom),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "target_iphone_simulator",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeIOSData, target_iphone_simulator),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "hw_machine",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeIOSData, hw_machine),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "system_version",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeIOSData, system_version),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "simulator_model_identifier",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeIOSData, simulator_model_identifier),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__native_iosdata__field_indices_by_name[] = {
  2,   /* field[2] = hw_machine */
  4,   /* field[4] = simulator_model_identifier */
  3,   /* field[3] = system_version */
  1,   /* field[1] = target_iphone_simulator */
  0,   /* field[0] = user_interface_idiom */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__native_iosdata__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_iosdata__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.NativeIOSData",
  "NativeIOSData",
  "Spotify__Clienttoken__Data__V0__NativeIOSData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__NativeIOSData),
  5,
  spotify__clienttoken__data__v0__native_iosdata__field_descriptors,
  spotify__clienttoken__data__v0__native_iosdata__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__native_iosdata__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__native_iosdata__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__native_desktop_windows_data__field_descriptors[8] =
{
  {
    "os_version",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, os_version),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "os_build",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, os_build),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "platform_id",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, platform_id),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_value_5",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, unknown_value_5),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_value_6",
    6,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, unknown_value_6),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "image_file_machine",
    7,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, image_file_machine),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "pe_machine",
    8,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, pe_machine),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_value_10",
    10,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_BOOL,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData, unknown_value_10),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__native_desktop_windows_data__field_indices_by_name[] = {
  5,   /* field[5] = image_file_machine */
  1,   /* field[1] = os_build */
  0,   /* field[0] = os_version */
  6,   /* field[6] = pe_machine */
  2,   /* field[2] = platform_id */
  7,   /* field[7] = unknown_value_10 */
  3,   /* field[3] = unknown_value_5 */
  4,   /* field[4] = unknown_value_6 */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__native_desktop_windows_data__number_ranges[3 + 1] =
{
  { 1, 0 },
  { 3, 1 },
  { 10, 7 },
  { 0, 8 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.NativeDesktopWindowsData",
  "NativeDesktopWindowsData",
  "Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData),
  8,
  spotify__clienttoken__data__v0__native_desktop_windows_data__field_descriptors,
  spotify__clienttoken__data__v0__native_desktop_windows_data__field_indices_by_name,
  3,  spotify__clienttoken__data__v0__native_desktop_windows_data__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__native_desktop_windows_data__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__native_desktop_linux_data__field_descriptors[4] =
{
  {
    "system_name",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData, system_name),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "system_release",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData, system_release),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "system_version",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData, system_version),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "hardware",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData, hardware),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__native_desktop_linux_data__field_indices_by_name[] = {
  3,   /* field[3] = hardware */
  0,   /* field[0] = system_name */
  1,   /* field[1] = system_release */
  2,   /* field[2] = system_version */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__native_desktop_linux_data__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.NativeDesktopLinuxData",
  "NativeDesktopLinuxData",
  "Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData),
  4,
  spotify__clienttoken__data__v0__native_desktop_linux_data__field_descriptors,
  spotify__clienttoken__data__v0__native_desktop_linux_data__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__native_desktop_linux_data__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__native_desktop_linux_data__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__native_desktop_mac_osdata__field_descriptors[3] =
{
  {
    "system_version",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData, system_version),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "hw_model",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData, hw_model),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "compiled_cpu_type",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData, compiled_cpu_type),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__native_desktop_mac_osdata__field_indices_by_name[] = {
  2,   /* field[2] = compiled_cpu_type */
  1,   /* field[1] = hw_model */
  0,   /* field[0] = system_version */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__native_desktop_mac_osdata__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.NativeDesktopMacOSData",
  "NativeDesktopMacOSData",
  "Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData),
  3,
  spotify__clienttoken__data__v0__native_desktop_mac_osdata__field_descriptors,
  spotify__clienttoken__data__v0__native_desktop_mac_osdata__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__native_desktop_mac_osdata__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__native_desktop_mac_osdata__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__clienttoken__data__v0__screen__field_descriptors[5] =
{
  {
    "width",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__Screen, width),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "height",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__Screen, height),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "density",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__Screen, density),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_value_4",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__Screen, unknown_value_4),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "unknown_value_5",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Clienttoken__Data__V0__Screen, unknown_value_5),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__clienttoken__data__v0__screen__field_indices_by_name[] = {
  2,   /* field[2] = density */
  1,   /* field[1] = height */
  3,   /* field[3] = unknown_value_4 */
  4,   /* field[4] = unknown_value_5 */
  0,   /* field[0] = width */
};
static const ProtobufCIntRange spotify__clienttoken__data__v0__screen__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__screen__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.clienttoken.data.v0.Screen",
  "Screen",
  "Spotify__Clienttoken__Data__V0__Screen",
  "spotify.clienttoken.data.v0",
  sizeof(Spotify__Clienttoken__Data__V0__Screen),
  5,
  spotify__clienttoken__data__v0__screen__field_descriptors,
  spotify__clienttoken__data__v0__screen__field_indices_by_name,
  1,  spotify__clienttoken__data__v0__screen__number_ranges,
  (ProtobufCMessageInit) spotify__clienttoken__data__v0__screen__init,
  NULL,NULL,NULL    /* reserved[123] */
};
