/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: entity_extension_data.proto */

#ifndef PROTOBUF_C_entity_5fextension_5fdata_2eproto__INCLUDED
#define PROTOBUF_C_entity_5fextension_5fdata_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif

#include "google_any.pb-c.h"

typedef struct Spotify__Extendedmetadata__EntityExtensionDataHeader Spotify__Extendedmetadata__EntityExtensionDataHeader;
typedef struct Spotify__Extendedmetadata__EntityExtensionData Spotify__Extendedmetadata__EntityExtensionData;
typedef struct Spotify__Extendedmetadata__PlainListAssoc Spotify__Extendedmetadata__PlainListAssoc;
typedef struct Spotify__Extendedmetadata__AssocHeader Spotify__Extendedmetadata__AssocHeader;
typedef struct Spotify__Extendedmetadata__Assoc Spotify__Extendedmetadata__Assoc;


/* --- enums --- */


/* --- messages --- */

struct  Spotify__Extendedmetadata__EntityExtensionDataHeader
{
  ProtobufCMessage base;
  int32_t status_code;
  char *etag;
  char *locale;
  int64_t cache_ttl_in_seconds;
  int64_t offline_ttl_in_seconds;
};
#define SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA_HEADER__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__entity_extension_data_header__descriptor) \
    , 0, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, 0, 0 }


struct  Spotify__Extendedmetadata__EntityExtensionData
{
  ProtobufCMessage base;
  Spotify__Extendedmetadata__EntityExtensionDataHeader *header;
  char *entity_uri;
  Google__Protobuf__Any *extension_data;
};
#define SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__entity_extension_data__descriptor) \
    , NULL, (char *)protobuf_c_empty_string, NULL }


struct  Spotify__Extendedmetadata__PlainListAssoc
{
  ProtobufCMessage base;
  size_t n_entity_uri;
  char **entity_uri;
};
#define SPOTIFY__EXTENDEDMETADATA__PLAIN_LIST_ASSOC__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__plain_list_assoc__descriptor) \
    , 0,NULL }


struct  Spotify__Extendedmetadata__AssocHeader
{
  ProtobufCMessage base;
};
#define SPOTIFY__EXTENDEDMETADATA__ASSOC_HEADER__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__assoc_header__descriptor) \
     }


struct  Spotify__Extendedmetadata__Assoc
{
  ProtobufCMessage base;
  Spotify__Extendedmetadata__AssocHeader *header;
  Spotify__Extendedmetadata__PlainListAssoc *plain_list;
};
#define SPOTIFY__EXTENDEDMETADATA__ASSOC__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__assoc__descriptor) \
    , NULL, NULL }


/* Spotify__Extendedmetadata__EntityExtensionDataHeader methods */
void   spotify__extendedmetadata__entity_extension_data_header__init
                     (Spotify__Extendedmetadata__EntityExtensionDataHeader         *message);
size_t spotify__extendedmetadata__entity_extension_data_header__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionDataHeader   *message);
size_t spotify__extendedmetadata__entity_extension_data_header__pack
                     (const Spotify__Extendedmetadata__EntityExtensionDataHeader   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__entity_extension_data_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionDataHeader   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__EntityExtensionDataHeader *
       spotify__extendedmetadata__entity_extension_data_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__entity_extension_data_header__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionDataHeader *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__EntityExtensionData methods */
void   spotify__extendedmetadata__entity_extension_data__init
                     (Spotify__Extendedmetadata__EntityExtensionData         *message);
size_t spotify__extendedmetadata__entity_extension_data__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionData   *message);
size_t spotify__extendedmetadata__entity_extension_data__pack
                     (const Spotify__Extendedmetadata__EntityExtensionData   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__entity_extension_data__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__EntityExtensionData *
       spotify__extendedmetadata__entity_extension_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__entity_extension_data__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__PlainListAssoc methods */
void   spotify__extendedmetadata__plain_list_assoc__init
                     (Spotify__Extendedmetadata__PlainListAssoc         *message);
size_t spotify__extendedmetadata__plain_list_assoc__get_packed_size
                     (const Spotify__Extendedmetadata__PlainListAssoc   *message);
size_t spotify__extendedmetadata__plain_list_assoc__pack
                     (const Spotify__Extendedmetadata__PlainListAssoc   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__plain_list_assoc__pack_to_buffer
                     (const Spotify__Extendedmetadata__PlainListAssoc   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__PlainListAssoc *
       spotify__extendedmetadata__plain_list_assoc__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__plain_list_assoc__free_unpacked
                     (Spotify__Extendedmetadata__PlainListAssoc *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__AssocHeader methods */
void   spotify__extendedmetadata__assoc_header__init
                     (Spotify__Extendedmetadata__AssocHeader         *message);
size_t spotify__extendedmetadata__assoc_header__get_packed_size
                     (const Spotify__Extendedmetadata__AssocHeader   *message);
size_t spotify__extendedmetadata__assoc_header__pack
                     (const Spotify__Extendedmetadata__AssocHeader   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__assoc_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__AssocHeader   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__AssocHeader *
       spotify__extendedmetadata__assoc_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__assoc_header__free_unpacked
                     (Spotify__Extendedmetadata__AssocHeader *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__Assoc methods */
void   spotify__extendedmetadata__assoc__init
                     (Spotify__Extendedmetadata__Assoc         *message);
size_t spotify__extendedmetadata__assoc__get_packed_size
                     (const Spotify__Extendedmetadata__Assoc   *message);
size_t spotify__extendedmetadata__assoc__pack
                     (const Spotify__Extendedmetadata__Assoc   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__assoc__pack_to_buffer
                     (const Spotify__Extendedmetadata__Assoc   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__Assoc *
       spotify__extendedmetadata__assoc__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__assoc__free_unpacked
                     (Spotify__Extendedmetadata__Assoc *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Extendedmetadata__EntityExtensionDataHeader_Closure)
                 (const Spotify__Extendedmetadata__EntityExtensionDataHeader *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__EntityExtensionData_Closure)
                 (const Spotify__Extendedmetadata__EntityExtensionData *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__PlainListAssoc_Closure)
                 (const Spotify__Extendedmetadata__PlainListAssoc *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__AssocHeader_Closure)
                 (const Spotify__Extendedmetadata__AssocHeader *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__Assoc_Closure)
                 (const Spotify__Extendedmetadata__Assoc *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data_header__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__plain_list_assoc__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__assoc_header__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__assoc__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_entity_5fextension_5fdata_2eproto__INCLUDED */
