/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: connectivity.proto */

#ifndef PROTOBUF_C_connectivity_2eproto__INCLUDED
#define PROTOBUF_C_connectivity_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif


typedef struct Spotify__Clienttoken__Data__V0__ConnectivitySdkData Spotify__Clienttoken__Data__V0__ConnectivitySdkData;
typedef struct Spotify__Clienttoken__Data__V0__PlatformSpecificData Spotify__Clienttoken__Data__V0__PlatformSpecificData;
typedef struct Spotify__Clienttoken__Data__V0__NativeAndroidData Spotify__Clienttoken__Data__V0__NativeAndroidData;
typedef struct Spotify__Clienttoken__Data__V0__NativeIOSData Spotify__Clienttoken__Data__V0__NativeIOSData;
typedef struct Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData;
typedef struct Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData;
typedef struct Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData;
typedef struct Spotify__Clienttoken__Data__V0__Screen Spotify__Clienttoken__Data__V0__Screen;


/* --- enums --- */


/* --- messages --- */

struct  Spotify__Clienttoken__Data__V0__ConnectivitySdkData
{
  ProtobufCMessage base;
  Spotify__Clienttoken__Data__V0__PlatformSpecificData *platform_specific_data;
  char *device_id;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__CONNECTIVITY_SDK_DATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor) \
    , NULL, (char *)protobuf_c_empty_string }


typedef enum {
  SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA__NOT_SET = 0,
  SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA_ANDROID = 1,
  SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA_IOS = 2,
  SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA_DESKTOP_MACOS = 3,
  SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA_DESKTOP_WINDOWS = 4,
  SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA_DESKTOP_LINUX = 5
    PROTOBUF_C__FORCE_ENUM_TO_BE_INT_SIZE(SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA__CASE)
} Spotify__Clienttoken__Data__V0__PlatformSpecificData__DataCase;

struct  Spotify__Clienttoken__Data__V0__PlatformSpecificData
{
  ProtobufCMessage base;
  Spotify__Clienttoken__Data__V0__PlatformSpecificData__DataCase data_case;
  union {
    Spotify__Clienttoken__Data__V0__NativeAndroidData *android;
    Spotify__Clienttoken__Data__V0__NativeIOSData *ios;
    Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *desktop_macos;
    Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *desktop_windows;
    Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *desktop_linux;
  };
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__platform_specific_data__descriptor) \
    , SPOTIFY__CLIENTTOKEN__DATA__V0__PLATFORM_SPECIFIC_DATA__DATA__NOT_SET, {0} }


struct  Spotify__Clienttoken__Data__V0__NativeAndroidData
{
  ProtobufCMessage base;
  Spotify__Clienttoken__Data__V0__Screen *screen_dimensions;
  char *android_version;
  int32_t api_version;
  char *device_name;
  char *model_str;
  char *vendor;
  char *vendor_2;
  int32_t unknown_value_8;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_ANDROID_DATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__native_android_data__descriptor) \
    , NULL, (char *)protobuf_c_empty_string, 0, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, 0 }


struct  Spotify__Clienttoken__Data__V0__NativeIOSData
{
  ProtobufCMessage base;
  /*
   * https://developer.apple.com/documentation/uikit/uiuserinterfaceidiom
   */
  int32_t user_interface_idiom;
  protobuf_c_boolean target_iphone_simulator;
  char *hw_machine;
  char *system_version;
  char *simulator_model_identifier;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_IOSDATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__native_iosdata__descriptor) \
    , 0, 0, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


struct  Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData
{
  ProtobufCMessage base;
  int32_t os_version;
  int32_t os_build;
  /*
   * https://docs.microsoft.com/en-us/dotnet/api/system.platformid?view=net-6.0
   */
  int32_t platform_id;
  int32_t unknown_value_5;
  int32_t unknown_value_6;
  /*
   * https://docs.microsoft.com/en-us/dotnet/api/system.reflection.imagefilemachine?view=net-6.0
   */
  int32_t image_file_machine;
  /*
   * https://docs.microsoft.com/en-us/dotnet/api/system.reflection.portableexecutable.machine?view=net-6.0
   */
  int32_t pe_machine;
  protobuf_c_boolean unknown_value_10;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_DESKTOP_WINDOWS_DATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor) \
    , 0, 0, 0, 0, 0, 0, 0, 0 }


struct  Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData
{
  ProtobufCMessage base;
  /*
   *  uname -s
   */
  char *system_name;
  /*
   *  -r
   */
  char *system_release;
  /*
   *  -v
   */
  char *system_version;
  /*
   *  -i
   */
  char *hardware;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_DESKTOP_LINUX_DATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


struct  Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData
{
  ProtobufCMessage base;
  char *system_version;
  char *hw_model;
  char *compiled_cpu_type;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__NATIVE_DESKTOP_MAC_OSDATA__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


struct  Spotify__Clienttoken__Data__V0__Screen
{
  ProtobufCMessage base;
  int32_t width;
  int32_t height;
  int32_t density;
  int32_t unknown_value_4;
  int32_t unknown_value_5;
};
#define SPOTIFY__CLIENTTOKEN__DATA__V0__SCREEN__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__clienttoken__data__v0__screen__descriptor) \
    , 0, 0, 0, 0, 0 }


/* Spotify__Clienttoken__Data__V0__ConnectivitySdkData methods */
void   spotify__clienttoken__data__v0__connectivity_sdk_data__init
                     (Spotify__Clienttoken__Data__V0__ConnectivitySdkData         *message);
size_t spotify__clienttoken__data__v0__connectivity_sdk_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData   *message);
size_t spotify__clienttoken__data__v0__connectivity_sdk_data__pack
                     (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__connectivity_sdk_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__ConnectivitySdkData *
       spotify__clienttoken__data__v0__connectivity_sdk_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__connectivity_sdk_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__ConnectivitySdkData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__PlatformSpecificData methods */
void   spotify__clienttoken__data__v0__platform_specific_data__init
                     (Spotify__Clienttoken__Data__V0__PlatformSpecificData         *message);
size_t spotify__clienttoken__data__v0__platform_specific_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__PlatformSpecificData   *message);
size_t spotify__clienttoken__data__v0__platform_specific_data__pack
                     (const Spotify__Clienttoken__Data__V0__PlatformSpecificData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__platform_specific_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__PlatformSpecificData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__PlatformSpecificData *
       spotify__clienttoken__data__v0__platform_specific_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__platform_specific_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__PlatformSpecificData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__NativeAndroidData methods */
void   spotify__clienttoken__data__v0__native_android_data__init
                     (Spotify__Clienttoken__Data__V0__NativeAndroidData         *message);
size_t spotify__clienttoken__data__v0__native_android_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeAndroidData   *message);
size_t spotify__clienttoken__data__v0__native_android_data__pack
                     (const Spotify__Clienttoken__Data__V0__NativeAndroidData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__native_android_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeAndroidData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__NativeAndroidData *
       spotify__clienttoken__data__v0__native_android_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__native_android_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeAndroidData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__NativeIOSData methods */
void   spotify__clienttoken__data__v0__native_iosdata__init
                     (Spotify__Clienttoken__Data__V0__NativeIOSData         *message);
size_t spotify__clienttoken__data__v0__native_iosdata__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeIOSData   *message);
size_t spotify__clienttoken__data__v0__native_iosdata__pack
                     (const Spotify__Clienttoken__Data__V0__NativeIOSData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__native_iosdata__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeIOSData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__NativeIOSData *
       spotify__clienttoken__data__v0__native_iosdata__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__native_iosdata__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeIOSData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData methods */
void   spotify__clienttoken__data__v0__native_desktop_windows_data__init
                     (Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData         *message);
size_t spotify__clienttoken__data__v0__native_desktop_windows_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData   *message);
size_t spotify__clienttoken__data__v0__native_desktop_windows_data__pack
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__native_desktop_windows_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *
       spotify__clienttoken__data__v0__native_desktop_windows_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__native_desktop_windows_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData methods */
void   spotify__clienttoken__data__v0__native_desktop_linux_data__init
                     (Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData         *message);
size_t spotify__clienttoken__data__v0__native_desktop_linux_data__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData   *message);
size_t spotify__clienttoken__data__v0__native_desktop_linux_data__pack
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__native_desktop_linux_data__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *
       spotify__clienttoken__data__v0__native_desktop_linux_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__native_desktop_linux_data__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData methods */
void   spotify__clienttoken__data__v0__native_desktop_mac_osdata__init
                     (Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData         *message);
size_t spotify__clienttoken__data__v0__native_desktop_mac_osdata__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData   *message);
size_t spotify__clienttoken__data__v0__native_desktop_mac_osdata__pack
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__native_desktop_mac_osdata__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *
       spotify__clienttoken__data__v0__native_desktop_mac_osdata__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__native_desktop_mac_osdata__free_unpacked
                     (Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Clienttoken__Data__V0__Screen methods */
void   spotify__clienttoken__data__v0__screen__init
                     (Spotify__Clienttoken__Data__V0__Screen         *message);
size_t spotify__clienttoken__data__v0__screen__get_packed_size
                     (const Spotify__Clienttoken__Data__V0__Screen   *message);
size_t spotify__clienttoken__data__v0__screen__pack
                     (const Spotify__Clienttoken__Data__V0__Screen   *message,
                      uint8_t             *out);
size_t spotify__clienttoken__data__v0__screen__pack_to_buffer
                     (const Spotify__Clienttoken__Data__V0__Screen   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Clienttoken__Data__V0__Screen *
       spotify__clienttoken__data__v0__screen__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__clienttoken__data__v0__screen__free_unpacked
                     (Spotify__Clienttoken__Data__V0__Screen *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Clienttoken__Data__V0__ConnectivitySdkData_Closure)
                 (const Spotify__Clienttoken__Data__V0__ConnectivitySdkData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__PlatformSpecificData_Closure)
                 (const Spotify__Clienttoken__Data__V0__PlatformSpecificData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__NativeAndroidData_Closure)
                 (const Spotify__Clienttoken__Data__V0__NativeAndroidData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__NativeIOSData_Closure)
                 (const Spotify__Clienttoken__Data__V0__NativeIOSData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData_Closure)
                 (const Spotify__Clienttoken__Data__V0__NativeDesktopWindowsData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData_Closure)
                 (const Spotify__Clienttoken__Data__V0__NativeDesktopLinuxData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData_Closure)
                 (const Spotify__Clienttoken__Data__V0__NativeDesktopMacOSData *message,
                  void *closure_data);
typedef void (*Spotify__Clienttoken__Data__V0__Screen_Closure)
                 (const Spotify__Clienttoken__Data__V0__Screen *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__connectivity_sdk_data__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__platform_specific_data__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_android_data__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_iosdata__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_desktop_windows_data__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_desktop_linux_data__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__native_desktop_mac_osdata__descriptor;
extern const ProtobufCMessageDescriptor spotify__clienttoken__data__v0__screen__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_connectivity_2eproto__INCLUDED */
