syntax = "proto3";

package spotify.download.proto;

message StorageResolveResponse {
  Result result = 1;
  enum Result {
    CDN = 0;
    STORAGE = 1;
    RESTRICTED = 3;
  }

  repeated string cdnurl = 2;
  bytes fileid = 4;
}
