/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: login5_user_info.proto */

#ifndef PROTOBUF_C_login5_5fuser_5finfo_2eproto__INCLUDED
#define PROTOBUF_C_login5_5fuser_5finfo_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif


typedef struct Spotify__Login5__V3__UserInfo Spotify__Login5__V3__UserInfo;


/* --- enums --- */

typedef enum _Spotify__Login5__V3__UserInfo__Gender {
  SPOTIFY__LOGIN5__V3__USER_INFO__GENDER__UNKNOWN = 0,
  SPOTIFY__LOGIN5__V3__USER_INFO__GENDER__MALE = 1,
  SPOTIFY__LOGIN5__V3__USER_INFO__GENDER__FEMALE = 2,
  SPOTIFY__LOGIN5__V3__USER_INFO__GENDER__NEUTRAL = 3
    PROTOBUF_C__FORCE_ENUM_TO_BE_INT_SIZE(SPOTIFY__LOGIN5__V3__USER_INFO__GENDER)
} Spotify__Login5__V3__UserInfo__Gender;

/* --- messages --- */

struct  Spotify__Login5__V3__UserInfo
{
  ProtobufCMessage base;
  char *name;
  char *email;
  protobuf_c_boolean email_verified;
  char *birthdate;
  Spotify__Login5__V3__UserInfo__Gender gender;
  char *phone_number;
  protobuf_c_boolean phone_number_verified;
  protobuf_c_boolean email_already_registered;
};
#define SPOTIFY__LOGIN5__V3__USER_INFO__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__user_info__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, 0, (char *)protobuf_c_empty_string, SPOTIFY__LOGIN5__V3__USER_INFO__GENDER__UNKNOWN, (char *)protobuf_c_empty_string, 0, 0 }


/* Spotify__Login5__V3__UserInfo methods */
void   spotify__login5__v3__user_info__init
                     (Spotify__Login5__V3__UserInfo         *message);
size_t spotify__login5__v3__user_info__get_packed_size
                     (const Spotify__Login5__V3__UserInfo   *message);
size_t spotify__login5__v3__user_info__pack
                     (const Spotify__Login5__V3__UserInfo   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__user_info__pack_to_buffer
                     (const Spotify__Login5__V3__UserInfo   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__UserInfo *
       spotify__login5__v3__user_info__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__user_info__free_unpacked
                     (Spotify__Login5__V3__UserInfo *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Login5__V3__UserInfo_Closure)
                 (const Spotify__Login5__V3__UserInfo *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__login5__v3__user_info__descriptor;
extern const ProtobufCEnumDescriptor    spotify__login5__v3__user_info__gender__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_login5_5fuser_5finfo_2eproto__INCLUDED */
