/* Generated by the protocol buffer compiler.  DO NOT EDIT! */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C_NO_DEPRECATED
#define PROTOBUF_C_NO_DEPRECATED
#endif

#include "cast_channel.v0.pb-c.h"
void   extensions__core_api__cast_channel__cast_message__init
                     (Extensions__CoreApi__CastChannel__CastMessage         *message)
{
  static Extensions__CoreApi__CastChannel__CastMessage init_value = EXTENSIONS__CORE_API__CAST_CHANNEL__CAST_MESSAGE__INIT;
  *message = init_value;
}
size_t extensions__core_api__cast_channel__cast_message__get_packed_size
                     (const Extensions__CoreApi__CastChannel__CastMessage *message)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__cast_message__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t extensions__core_api__cast_channel__cast_message__pack
                     (const Extensions__CoreApi__CastChannel__CastMessage *message,
                      uint8_t       *out)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__cast_message__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t extensions__core_api__cast_channel__cast_message__pack_to_buffer
                     (const Extensions__CoreApi__CastChannel__CastMessage *message,
                      ProtobufCBuffer *buffer)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__cast_message__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Extensions__CoreApi__CastChannel__CastMessage *
       extensions__core_api__cast_channel__cast_message__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Extensions__CoreApi__CastChannel__CastMessage *)
     protobuf_c_message_unpack (&extensions__core_api__cast_channel__cast_message__descriptor,
                                allocator, len, data);
}
void   extensions__core_api__cast_channel__cast_message__free_unpacked
                     (Extensions__CoreApi__CastChannel__CastMessage *message,
                      ProtobufCAllocator *allocator)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__cast_message__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   extensions__core_api__cast_channel__auth_challenge__init
                     (Extensions__CoreApi__CastChannel__AuthChallenge         *message)
{
  static Extensions__CoreApi__CastChannel__AuthChallenge init_value = EXTENSIONS__CORE_API__CAST_CHANNEL__AUTH_CHALLENGE__INIT;
  *message = init_value;
}
size_t extensions__core_api__cast_channel__auth_challenge__get_packed_size
                     (const Extensions__CoreApi__CastChannel__AuthChallenge *message)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_challenge__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t extensions__core_api__cast_channel__auth_challenge__pack
                     (const Extensions__CoreApi__CastChannel__AuthChallenge *message,
                      uint8_t       *out)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_challenge__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t extensions__core_api__cast_channel__auth_challenge__pack_to_buffer
                     (const Extensions__CoreApi__CastChannel__AuthChallenge *message,
                      ProtobufCBuffer *buffer)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_challenge__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Extensions__CoreApi__CastChannel__AuthChallenge *
       extensions__core_api__cast_channel__auth_challenge__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Extensions__CoreApi__CastChannel__AuthChallenge *)
     protobuf_c_message_unpack (&extensions__core_api__cast_channel__auth_challenge__descriptor,
                                allocator, len, data);
}
void   extensions__core_api__cast_channel__auth_challenge__free_unpacked
                     (Extensions__CoreApi__CastChannel__AuthChallenge *message,
                      ProtobufCAllocator *allocator)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_challenge__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   extensions__core_api__cast_channel__auth_response__init
                     (Extensions__CoreApi__CastChannel__AuthResponse         *message)
{
  static Extensions__CoreApi__CastChannel__AuthResponse init_value = EXTENSIONS__CORE_API__CAST_CHANNEL__AUTH_RESPONSE__INIT;
  *message = init_value;
}
size_t extensions__core_api__cast_channel__auth_response__get_packed_size
                     (const Extensions__CoreApi__CastChannel__AuthResponse *message)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_response__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t extensions__core_api__cast_channel__auth_response__pack
                     (const Extensions__CoreApi__CastChannel__AuthResponse *message,
                      uint8_t       *out)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_response__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t extensions__core_api__cast_channel__auth_response__pack_to_buffer
                     (const Extensions__CoreApi__CastChannel__AuthResponse *message,
                      ProtobufCBuffer *buffer)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_response__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Extensions__CoreApi__CastChannel__AuthResponse *
       extensions__core_api__cast_channel__auth_response__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Extensions__CoreApi__CastChannel__AuthResponse *)
     protobuf_c_message_unpack (&extensions__core_api__cast_channel__auth_response__descriptor,
                                allocator, len, data);
}
void   extensions__core_api__cast_channel__auth_response__free_unpacked
                     (Extensions__CoreApi__CastChannel__AuthResponse *message,
                      ProtobufCAllocator *allocator)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_response__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   extensions__core_api__cast_channel__auth_error__init
                     (Extensions__CoreApi__CastChannel__AuthError         *message)
{
  static Extensions__CoreApi__CastChannel__AuthError init_value = EXTENSIONS__CORE_API__CAST_CHANNEL__AUTH_ERROR__INIT;
  *message = init_value;
}
size_t extensions__core_api__cast_channel__auth_error__get_packed_size
                     (const Extensions__CoreApi__CastChannel__AuthError *message)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_error__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t extensions__core_api__cast_channel__auth_error__pack
                     (const Extensions__CoreApi__CastChannel__AuthError *message,
                      uint8_t       *out)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_error__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t extensions__core_api__cast_channel__auth_error__pack_to_buffer
                     (const Extensions__CoreApi__CastChannel__AuthError *message,
                      ProtobufCBuffer *buffer)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_error__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Extensions__CoreApi__CastChannel__AuthError *
       extensions__core_api__cast_channel__auth_error__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Extensions__CoreApi__CastChannel__AuthError *)
     protobuf_c_message_unpack (&extensions__core_api__cast_channel__auth_error__descriptor,
                                allocator, len, data);
}
void   extensions__core_api__cast_channel__auth_error__free_unpacked
                     (Extensions__CoreApi__CastChannel__AuthError *message,
                      ProtobufCAllocator *allocator)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__auth_error__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   extensions__core_api__cast_channel__device_auth_message__init
                     (Extensions__CoreApi__CastChannel__DeviceAuthMessage         *message)
{
  static Extensions__CoreApi__CastChannel__DeviceAuthMessage init_value = EXTENSIONS__CORE_API__CAST_CHANNEL__DEVICE_AUTH_MESSAGE__INIT;
  *message = init_value;
}
size_t extensions__core_api__cast_channel__device_auth_message__get_packed_size
                     (const Extensions__CoreApi__CastChannel__DeviceAuthMessage *message)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__device_auth_message__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t extensions__core_api__cast_channel__device_auth_message__pack
                     (const Extensions__CoreApi__CastChannel__DeviceAuthMessage *message,
                      uint8_t       *out)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__device_auth_message__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t extensions__core_api__cast_channel__device_auth_message__pack_to_buffer
                     (const Extensions__CoreApi__CastChannel__DeviceAuthMessage *message,
                      ProtobufCBuffer *buffer)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__device_auth_message__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Extensions__CoreApi__CastChannel__DeviceAuthMessage *
       extensions__core_api__cast_channel__device_auth_message__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Extensions__CoreApi__CastChannel__DeviceAuthMessage *)
     protobuf_c_message_unpack (&extensions__core_api__cast_channel__device_auth_message__descriptor,
                                allocator, len, data);
}
void   extensions__core_api__cast_channel__device_auth_message__free_unpacked
                     (Extensions__CoreApi__CastChannel__DeviceAuthMessage *message,
                      ProtobufCAllocator *allocator)
{
  PROTOBUF_C_ASSERT (message->base.descriptor == &extensions__core_api__cast_channel__device_auth_message__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
const ProtobufCEnumValue extensions__core_api__cast_channel__cast_message__protocol_version__enum_values_by_number[1] =
{
  { "CASTV2_1_0", "EXTENSIONS__CORE_API__CAST_CHANNEL__CAST_MESSAGE__PROTOCOL_VERSION__CASTV2_1_0", 0 },
};
static const ProtobufCIntRange extensions__core_api__cast_channel__cast_message__protocol_version__value_ranges[] = {
{0, 0},{0, 1}
};
const ProtobufCEnumValueIndex extensions__core_api__cast_channel__cast_message__protocol_version__enum_values_by_name[1] =
{
  { "CASTV2_1_0", 0 },
};
const ProtobufCEnumDescriptor extensions__core_api__cast_channel__cast_message__protocol_version__descriptor =
{
  PROTOBUF_C_ENUM_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.CastMessage.ProtocolVersion",
  "ProtocolVersion",
  "Extensions__CoreApi__CastChannel__CastMessage__ProtocolVersion",
  "extensions.core_api.cast_channel",
  1,
  extensions__core_api__cast_channel__cast_message__protocol_version__enum_values_by_number,
  1,
  extensions__core_api__cast_channel__cast_message__protocol_version__enum_values_by_name,
  1,
  extensions__core_api__cast_channel__cast_message__protocol_version__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
const ProtobufCEnumValue extensions__core_api__cast_channel__cast_message__payload_type__enum_values_by_number[2] =
{
  { "STRING", "EXTENSIONS__CORE_API__CAST_CHANNEL__CAST_MESSAGE__PAYLOAD_TYPE__STRING", 0 },
  { "BINARY", "EXTENSIONS__CORE_API__CAST_CHANNEL__CAST_MESSAGE__PAYLOAD_TYPE__BINARY", 1 },
};
static const ProtobufCIntRange extensions__core_api__cast_channel__cast_message__payload_type__value_ranges[] = {
{0, 0},{0, 2}
};
const ProtobufCEnumValueIndex extensions__core_api__cast_channel__cast_message__payload_type__enum_values_by_name[2] =
{
  { "BINARY", 1 },
  { "STRING", 0 },
};
const ProtobufCEnumDescriptor extensions__core_api__cast_channel__cast_message__payload_type__descriptor =
{
  PROTOBUF_C_ENUM_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.CastMessage.PayloadType",
  "PayloadType",
  "Extensions__CoreApi__CastChannel__CastMessage__PayloadType",
  "extensions.core_api.cast_channel",
  2,
  extensions__core_api__cast_channel__cast_message__payload_type__enum_values_by_number,
  2,
  extensions__core_api__cast_channel__cast_message__payload_type__enum_values_by_name,
  1,
  extensions__core_api__cast_channel__cast_message__payload_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCFieldDescriptor extensions__core_api__cast_channel__cast_message__field_descriptors[7] =
{
  {
    "protocol_version",
    1,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, protocol_version),
    &extensions__core_api__cast_channel__cast_message__protocol_version__descriptor,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "source_id",
    2,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, source_id),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "destination_id",
    3,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, destination_id),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "namespace",
    4,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, namespace_),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "payload_type",
    5,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, payload_type),
    &extensions__core_api__cast_channel__cast_message__payload_type__descriptor,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "payload_utf8",
    6,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, payload_utf8),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "payload_binary",
    7,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_BYTES,
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, has_payload_binary),
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__CastMessage, payload_binary),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned extensions__core_api__cast_channel__cast_message__field_indices_by_name[] = {
  2,   /* field[2] = destination_id */
  3,   /* field[3] = namespace */
  6,   /* field[6] = payload_binary */
  4,   /* field[4] = payload_type */
  5,   /* field[5] = payload_utf8 */
  0,   /* field[0] = protocol_version */
  1,   /* field[1] = source_id */
};
static const ProtobufCIntRange extensions__core_api__cast_channel__cast_message__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 7 }
};
const ProtobufCMessageDescriptor extensions__core_api__cast_channel__cast_message__descriptor =
{
  PROTOBUF_C_MESSAGE_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.CastMessage",
  "CastMessage",
  "Extensions__CoreApi__CastChannel__CastMessage",
  "extensions.core_api.cast_channel",
  sizeof(Extensions__CoreApi__CastChannel__CastMessage),
  7,
  extensions__core_api__cast_channel__cast_message__field_descriptors,
  extensions__core_api__cast_channel__cast_message__field_indices_by_name,
  1,  extensions__core_api__cast_channel__cast_message__number_ranges,
  (ProtobufCMessageInit) extensions__core_api__cast_channel__cast_message__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const Extensions__CoreApi__CastChannel__SignatureAlgorithm extensions__core_api__cast_channel__auth_challenge__signature_algorithm__default_value = EXTENSIONS__CORE_API__CAST_CHANNEL__SIGNATURE_ALGORITHM__RSASSA_PKCS1V15;
static const ProtobufCFieldDescriptor extensions__core_api__cast_channel__auth_challenge__field_descriptors[1] =
{
  {
    "signature_algorithm",
    1,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_ENUM,
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthChallenge, has_signature_algorithm),
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthChallenge, signature_algorithm),
    &extensions__core_api__cast_channel__signature_algorithm__descriptor,
    &extensions__core_api__cast_channel__auth_challenge__signature_algorithm__default_value,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned extensions__core_api__cast_channel__auth_challenge__field_indices_by_name[] = {
  0,   /* field[0] = signature_algorithm */
};
static const ProtobufCIntRange extensions__core_api__cast_channel__auth_challenge__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor extensions__core_api__cast_channel__auth_challenge__descriptor =
{
  PROTOBUF_C_MESSAGE_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.AuthChallenge",
  "AuthChallenge",
  "Extensions__CoreApi__CastChannel__AuthChallenge",
  "extensions.core_api.cast_channel",
  sizeof(Extensions__CoreApi__CastChannel__AuthChallenge),
  1,
  extensions__core_api__cast_channel__auth_challenge__field_descriptors,
  extensions__core_api__cast_channel__auth_challenge__field_indices_by_name,
  1,  extensions__core_api__cast_channel__auth_challenge__number_ranges,
  (ProtobufCMessageInit) extensions__core_api__cast_channel__auth_challenge__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const Extensions__CoreApi__CastChannel__SignatureAlgorithm extensions__core_api__cast_channel__auth_response__signature_algorithm__default_value = EXTENSIONS__CORE_API__CAST_CHANNEL__SIGNATURE_ALGORITHM__RSASSA_PKCS1V15;
static const ProtobufCFieldDescriptor extensions__core_api__cast_channel__auth_response__field_descriptors[4] =
{
  {
    "signature",
    1,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_BYTES,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthResponse, signature),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "client_auth_certificate",
    2,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_BYTES,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthResponse, client_auth_certificate),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "intermediate_certificate",
    3,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_BYTES,
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthResponse, n_intermediate_certificate),
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthResponse, intermediate_certificate),
    NULL,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "signature_algorithm",
    4,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_ENUM,
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthResponse, has_signature_algorithm),
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthResponse, signature_algorithm),
    &extensions__core_api__cast_channel__signature_algorithm__descriptor,
    &extensions__core_api__cast_channel__auth_response__signature_algorithm__default_value,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned extensions__core_api__cast_channel__auth_response__field_indices_by_name[] = {
  1,   /* field[1] = client_auth_certificate */
  2,   /* field[2] = intermediate_certificate */
  0,   /* field[0] = signature */
  3,   /* field[3] = signature_algorithm */
};
static const ProtobufCIntRange extensions__core_api__cast_channel__auth_response__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor extensions__core_api__cast_channel__auth_response__descriptor =
{
  PROTOBUF_C_MESSAGE_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.AuthResponse",
  "AuthResponse",
  "Extensions__CoreApi__CastChannel__AuthResponse",
  "extensions.core_api.cast_channel",
  sizeof(Extensions__CoreApi__CastChannel__AuthResponse),
  4,
  extensions__core_api__cast_channel__auth_response__field_descriptors,
  extensions__core_api__cast_channel__auth_response__field_indices_by_name,
  1,  extensions__core_api__cast_channel__auth_response__number_ranges,
  (ProtobufCMessageInit) extensions__core_api__cast_channel__auth_response__init,
  NULL,NULL,NULL    /* reserved[123] */
};
const ProtobufCEnumValue extensions__core_api__cast_channel__auth_error__error_type__enum_values_by_number[3] =
{
  { "INTERNAL_ERROR", "EXTENSIONS__CORE_API__CAST_CHANNEL__AUTH_ERROR__ERROR_TYPE__INTERNAL_ERROR", 0 },
  { "NO_TLS", "EXTENSIONS__CORE_API__CAST_CHANNEL__AUTH_ERROR__ERROR_TYPE__NO_TLS", 1 },
  { "SIGNATURE_ALGORITHM_UNAVAILABLE", "EXTENSIONS__CORE_API__CAST_CHANNEL__AUTH_ERROR__ERROR_TYPE__SIGNATURE_ALGORITHM_UNAVAILABLE", 2 },
};
static const ProtobufCIntRange extensions__core_api__cast_channel__auth_error__error_type__value_ranges[] = {
{0, 0},{0, 3}
};
const ProtobufCEnumValueIndex extensions__core_api__cast_channel__auth_error__error_type__enum_values_by_name[3] =
{
  { "INTERNAL_ERROR", 0 },
  { "NO_TLS", 1 },
  { "SIGNATURE_ALGORITHM_UNAVAILABLE", 2 },
};
const ProtobufCEnumDescriptor extensions__core_api__cast_channel__auth_error__error_type__descriptor =
{
  PROTOBUF_C_ENUM_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.AuthError.ErrorType",
  "ErrorType",
  "Extensions__CoreApi__CastChannel__AuthError__ErrorType",
  "extensions.core_api.cast_channel",
  3,
  extensions__core_api__cast_channel__auth_error__error_type__enum_values_by_number,
  3,
  extensions__core_api__cast_channel__auth_error__error_type__enum_values_by_name,
  1,
  extensions__core_api__cast_channel__auth_error__error_type__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCFieldDescriptor extensions__core_api__cast_channel__auth_error__field_descriptors[1] =
{
  {
    "error_type",
    1,
    PROTOBUF_C_LABEL_REQUIRED,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__AuthError, error_type),
    &extensions__core_api__cast_channel__auth_error__error_type__descriptor,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned extensions__core_api__cast_channel__auth_error__field_indices_by_name[] = {
  0,   /* field[0] = error_type */
};
static const ProtobufCIntRange extensions__core_api__cast_channel__auth_error__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor extensions__core_api__cast_channel__auth_error__descriptor =
{
  PROTOBUF_C_MESSAGE_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.AuthError",
  "AuthError",
  "Extensions__CoreApi__CastChannel__AuthError",
  "extensions.core_api.cast_channel",
  sizeof(Extensions__CoreApi__CastChannel__AuthError),
  1,
  extensions__core_api__cast_channel__auth_error__field_descriptors,
  extensions__core_api__cast_channel__auth_error__field_indices_by_name,
  1,  extensions__core_api__cast_channel__auth_error__number_ranges,
  (ProtobufCMessageInit) extensions__core_api__cast_channel__auth_error__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor extensions__core_api__cast_channel__device_auth_message__field_descriptors[3] =
{
  {
    "challenge",
    1,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__DeviceAuthMessage, challenge),
    &extensions__core_api__cast_channel__auth_challenge__descriptor,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "response",
    2,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__DeviceAuthMessage, response),
    &extensions__core_api__cast_channel__auth_response__descriptor,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "error",
    3,
    PROTOBUF_C_LABEL_OPTIONAL,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    PROTOBUF_C_OFFSETOF(Extensions__CoreApi__CastChannel__DeviceAuthMessage, error),
    &extensions__core_api__cast_channel__auth_error__descriptor,
    NULL,
    0,            /* packed */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned extensions__core_api__cast_channel__device_auth_message__field_indices_by_name[] = {
  0,   /* field[0] = challenge */
  2,   /* field[2] = error */
  1,   /* field[1] = response */
};
static const ProtobufCIntRange extensions__core_api__cast_channel__device_auth_message__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor extensions__core_api__cast_channel__device_auth_message__descriptor =
{
  PROTOBUF_C_MESSAGE_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.DeviceAuthMessage",
  "DeviceAuthMessage",
  "Extensions__CoreApi__CastChannel__DeviceAuthMessage",
  "extensions.core_api.cast_channel",
  sizeof(Extensions__CoreApi__CastChannel__DeviceAuthMessage),
  3,
  extensions__core_api__cast_channel__device_auth_message__field_descriptors,
  extensions__core_api__cast_channel__device_auth_message__field_indices_by_name,
  1,  extensions__core_api__cast_channel__device_auth_message__number_ranges,
  (ProtobufCMessageInit) extensions__core_api__cast_channel__device_auth_message__init,
  NULL,NULL,NULL    /* reserved[123] */
};
const ProtobufCEnumValue extensions__core_api__cast_channel__signature_algorithm__enum_values_by_number[3] =
{
  { "UNSPECIFIED", "EXTENSIONS__CORE_API__CAST_CHANNEL__SIGNATURE_ALGORITHM__UNSPECIFIED", 0 },
  { "RSASSA_PKCS1v15", "EXTENSIONS__CORE_API__CAST_CHANNEL__SIGNATURE_ALGORITHM__RSASSA_PKCS1V15", 1 },
  { "RSASSA_PSS", "EXTENSIONS__CORE_API__CAST_CHANNEL__SIGNATURE_ALGORITHM__RSASSA_PSS", 2 },
};
static const ProtobufCIntRange extensions__core_api__cast_channel__signature_algorithm__value_ranges[] = {
{0, 0},{0, 3}
};
const ProtobufCEnumValueIndex extensions__core_api__cast_channel__signature_algorithm__enum_values_by_name[3] =
{
  { "RSASSA_PKCS1v15", 1 },
  { "RSASSA_PSS", 2 },
  { "UNSPECIFIED", 0 },
};
const ProtobufCEnumDescriptor extensions__core_api__cast_channel__signature_algorithm__descriptor =
{
  PROTOBUF_C_ENUM_DESCRIPTOR_MAGIC,
  "extensions.core_api.cast_channel.SignatureAlgorithm",
  "SignatureAlgorithm",
  "Extensions__CoreApi__CastChannel__SignatureAlgorithm",
  "extensions.core_api.cast_channel",
  3,
  extensions__core_api__cast_channel__signature_algorithm__enum_values_by_number,
  3,
  extensions__core_api__cast_channel__signature_algorithm__enum_values_by_name,
  1,
  extensions__core_api__cast_channel__signature_algorithm__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
