/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: extended_metadata.proto */

#ifndef PROTOBUF_C_extended_5fmetadata_2eproto__INCLUDED
#define PROTOBUF_C_extended_5fmetadata_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif

#include "entity_extension_data.pb-c.h"
#include "extension_kind.pb-c.h"

typedef struct Spotify__Extendedmetadata__ExtensionQuery Spotify__Extendedmetadata__ExtensionQuery;
typedef struct Spotify__Extendedmetadata__EntityRequest Spotify__Extendedmetadata__EntityRequest;
typedef struct Spotify__Extendedmetadata__BatchedEntityRequestHeader Spotify__Extendedmetadata__BatchedEntityRequestHeader;
typedef struct Spotify__Extendedmetadata__BatchedEntityRequest Spotify__Extendedmetadata__BatchedEntityRequest;
typedef struct Spotify__Extendedmetadata__EntityExtensionDataArrayHeader Spotify__Extendedmetadata__EntityExtensionDataArrayHeader;
typedef struct Spotify__Extendedmetadata__EntityExtensionDataArray Spotify__Extendedmetadata__EntityExtensionDataArray;
typedef struct Spotify__Extendedmetadata__BatchedExtensionResponseHeader Spotify__Extendedmetadata__BatchedExtensionResponseHeader;
typedef struct Spotify__Extendedmetadata__BatchedExtensionResponse Spotify__Extendedmetadata__BatchedExtensionResponse;


/* --- enums --- */

typedef enum _Spotify__Extendedmetadata__ExtensionType {
  SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__UNKNOWN = 0,
  SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__GENERIC = 1,
  SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__ASSOC = 2
    PROTOBUF_C__FORCE_ENUM_TO_BE_INT_SIZE(SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE)
} Spotify__Extendedmetadata__ExtensionType;

/* --- messages --- */

struct  Spotify__Extendedmetadata__ExtensionQuery
{
  ProtobufCMessage base;
  Spotify__Extendedmetadata__ExtensionKind extension_kind;
  char *etag;
};
#define SPOTIFY__EXTENDEDMETADATA__EXTENSION_QUERY__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__extension_query__descriptor) \
    , SPOTIFY__EXTENDEDMETADATA__EXTENSION_KIND__UNKNOWN_EXTENSION, (char *)protobuf_c_empty_string }


struct  Spotify__Extendedmetadata__EntityRequest
{
  ProtobufCMessage base;
  char *entity_uri;
  size_t n_query;
  Spotify__Extendedmetadata__ExtensionQuery **query;
};
#define SPOTIFY__EXTENDEDMETADATA__ENTITY_REQUEST__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__entity_request__descriptor) \
    , (char *)protobuf_c_empty_string, 0,NULL }


struct  Spotify__Extendedmetadata__BatchedEntityRequestHeader
{
  ProtobufCMessage base;
  char *country;
  char *catalogue;
  ProtobufCBinaryData task_id;
};
#define SPOTIFY__EXTENDEDMETADATA__BATCHED_ENTITY_REQUEST_HEADER__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__batched_entity_request_header__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, {0,NULL} }


struct  Spotify__Extendedmetadata__BatchedEntityRequest
{
  ProtobufCMessage base;
  Spotify__Extendedmetadata__BatchedEntityRequestHeader *header;
  size_t n_entity_request;
  Spotify__Extendedmetadata__EntityRequest **entity_request;
};
#define SPOTIFY__EXTENDEDMETADATA__BATCHED_ENTITY_REQUEST__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__batched_entity_request__descriptor) \
    , NULL, 0,NULL }


struct  Spotify__Extendedmetadata__EntityExtensionDataArrayHeader
{
  ProtobufCMessage base;
  int32_t provider_error_status;
  int64_t cache_ttl_in_seconds;
  int64_t offline_ttl_in_seconds;
  Spotify__Extendedmetadata__ExtensionType extension_type;
};
#define SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA_ARRAY_HEADER__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__entity_extension_data_array_header__descriptor) \
    , 0, 0, 0, SPOTIFY__EXTENDEDMETADATA__EXTENSION_TYPE__UNKNOWN }


struct  Spotify__Extendedmetadata__EntityExtensionDataArray
{
  ProtobufCMessage base;
  Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *header;
  Spotify__Extendedmetadata__ExtensionKind extension_kind;
  size_t n_extension_data;
  Spotify__Extendedmetadata__EntityExtensionData **extension_data;
};
#define SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA_ARRAY__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__entity_extension_data_array__descriptor) \
    , NULL, SPOTIFY__EXTENDEDMETADATA__EXTENSION_KIND__UNKNOWN_EXTENSION, 0,NULL }


struct  Spotify__Extendedmetadata__BatchedExtensionResponseHeader
{
  ProtobufCMessage base;
};
#define SPOTIFY__EXTENDEDMETADATA__BATCHED_EXTENSION_RESPONSE_HEADER__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__batched_extension_response_header__descriptor) \
     }


struct  Spotify__Extendedmetadata__BatchedExtensionResponse
{
  ProtobufCMessage base;
  Spotify__Extendedmetadata__BatchedExtensionResponseHeader *header;
  size_t n_extended_metadata;
  Spotify__Extendedmetadata__EntityExtensionDataArray **extended_metadata;
};
#define SPOTIFY__EXTENDEDMETADATA__BATCHED_EXTENSION_RESPONSE__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__extendedmetadata__batched_extension_response__descriptor) \
    , NULL, 0,NULL }


/* Spotify__Extendedmetadata__ExtensionQuery methods */
void   spotify__extendedmetadata__extension_query__init
                     (Spotify__Extendedmetadata__ExtensionQuery         *message);
size_t spotify__extendedmetadata__extension_query__get_packed_size
                     (const Spotify__Extendedmetadata__ExtensionQuery   *message);
size_t spotify__extendedmetadata__extension_query__pack
                     (const Spotify__Extendedmetadata__ExtensionQuery   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__extension_query__pack_to_buffer
                     (const Spotify__Extendedmetadata__ExtensionQuery   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__ExtensionQuery *
       spotify__extendedmetadata__extension_query__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__extension_query__free_unpacked
                     (Spotify__Extendedmetadata__ExtensionQuery *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__EntityRequest methods */
void   spotify__extendedmetadata__entity_request__init
                     (Spotify__Extendedmetadata__EntityRequest         *message);
size_t spotify__extendedmetadata__entity_request__get_packed_size
                     (const Spotify__Extendedmetadata__EntityRequest   *message);
size_t spotify__extendedmetadata__entity_request__pack
                     (const Spotify__Extendedmetadata__EntityRequest   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__entity_request__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityRequest   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__EntityRequest *
       spotify__extendedmetadata__entity_request__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__entity_request__free_unpacked
                     (Spotify__Extendedmetadata__EntityRequest *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__BatchedEntityRequestHeader methods */
void   spotify__extendedmetadata__batched_entity_request_header__init
                     (Spotify__Extendedmetadata__BatchedEntityRequestHeader         *message);
size_t spotify__extendedmetadata__batched_entity_request_header__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedEntityRequestHeader   *message);
size_t spotify__extendedmetadata__batched_entity_request_header__pack
                     (const Spotify__Extendedmetadata__BatchedEntityRequestHeader   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__batched_entity_request_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedEntityRequestHeader   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__BatchedEntityRequestHeader *
       spotify__extendedmetadata__batched_entity_request_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__batched_entity_request_header__free_unpacked
                     (Spotify__Extendedmetadata__BatchedEntityRequestHeader *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__BatchedEntityRequest methods */
void   spotify__extendedmetadata__batched_entity_request__init
                     (Spotify__Extendedmetadata__BatchedEntityRequest         *message);
size_t spotify__extendedmetadata__batched_entity_request__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedEntityRequest   *message);
size_t spotify__extendedmetadata__batched_entity_request__pack
                     (const Spotify__Extendedmetadata__BatchedEntityRequest   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__batched_entity_request__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedEntityRequest   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__BatchedEntityRequest *
       spotify__extendedmetadata__batched_entity_request__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__batched_entity_request__free_unpacked
                     (Spotify__Extendedmetadata__BatchedEntityRequest *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__EntityExtensionDataArrayHeader methods */
void   spotify__extendedmetadata__entity_extension_data_array_header__init
                     (Spotify__Extendedmetadata__EntityExtensionDataArrayHeader         *message);
size_t spotify__extendedmetadata__entity_extension_data_array_header__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader   *message);
size_t spotify__extendedmetadata__entity_extension_data_array_header__pack
                     (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__entity_extension_data_array_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *
       spotify__extendedmetadata__entity_extension_data_array_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__entity_extension_data_array_header__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__EntityExtensionDataArray methods */
void   spotify__extendedmetadata__entity_extension_data_array__init
                     (Spotify__Extendedmetadata__EntityExtensionDataArray         *message);
size_t spotify__extendedmetadata__entity_extension_data_array__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionDataArray   *message);
size_t spotify__extendedmetadata__entity_extension_data_array__pack
                     (const Spotify__Extendedmetadata__EntityExtensionDataArray   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__entity_extension_data_array__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionDataArray   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__EntityExtensionDataArray *
       spotify__extendedmetadata__entity_extension_data_array__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__entity_extension_data_array__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionDataArray *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__BatchedExtensionResponseHeader methods */
void   spotify__extendedmetadata__batched_extension_response_header__init
                     (Spotify__Extendedmetadata__BatchedExtensionResponseHeader         *message);
size_t spotify__extendedmetadata__batched_extension_response_header__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader   *message);
size_t spotify__extendedmetadata__batched_extension_response_header__pack
                     (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__batched_extension_response_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__BatchedExtensionResponseHeader *
       spotify__extendedmetadata__batched_extension_response_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__batched_extension_response_header__free_unpacked
                     (Spotify__Extendedmetadata__BatchedExtensionResponseHeader *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Extendedmetadata__BatchedExtensionResponse methods */
void   spotify__extendedmetadata__batched_extension_response__init
                     (Spotify__Extendedmetadata__BatchedExtensionResponse         *message);
size_t spotify__extendedmetadata__batched_extension_response__get_packed_size
                     (const Spotify__Extendedmetadata__BatchedExtensionResponse   *message);
size_t spotify__extendedmetadata__batched_extension_response__pack
                     (const Spotify__Extendedmetadata__BatchedExtensionResponse   *message,
                      uint8_t             *out);
size_t spotify__extendedmetadata__batched_extension_response__pack_to_buffer
                     (const Spotify__Extendedmetadata__BatchedExtensionResponse   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Extendedmetadata__BatchedExtensionResponse *
       spotify__extendedmetadata__batched_extension_response__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__extendedmetadata__batched_extension_response__free_unpacked
                     (Spotify__Extendedmetadata__BatchedExtensionResponse *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Extendedmetadata__ExtensionQuery_Closure)
                 (const Spotify__Extendedmetadata__ExtensionQuery *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__EntityRequest_Closure)
                 (const Spotify__Extendedmetadata__EntityRequest *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__BatchedEntityRequestHeader_Closure)
                 (const Spotify__Extendedmetadata__BatchedEntityRequestHeader *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__BatchedEntityRequest_Closure)
                 (const Spotify__Extendedmetadata__BatchedEntityRequest *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__EntityExtensionDataArrayHeader_Closure)
                 (const Spotify__Extendedmetadata__EntityExtensionDataArrayHeader *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__EntityExtensionDataArray_Closure)
                 (const Spotify__Extendedmetadata__EntityExtensionDataArray *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__BatchedExtensionResponseHeader_Closure)
                 (const Spotify__Extendedmetadata__BatchedExtensionResponseHeader *message,
                  void *closure_data);
typedef void (*Spotify__Extendedmetadata__BatchedExtensionResponse_Closure)
                 (const Spotify__Extendedmetadata__BatchedExtensionResponse *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCEnumDescriptor    spotify__extendedmetadata__extension_type__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__extension_query__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_request__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_entity_request_header__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_entity_request__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data_array_header__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data_array__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_extension_response_header__descriptor;
extern const ProtobufCMessageDescriptor spotify__extendedmetadata__batched_extension_response__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_extended_5fmetadata_2eproto__INCLUDED */
