/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: login5_credentials.proto */

#ifndef PROTOBUF_C_login5_5fcredentials_2eproto__INCLUDED
#define PROTOBUF_C_login5_5fcredentials_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif


typedef struct Spotify__Login5__V3__Credentials__StoredCredential Spotify__Login5__V3__Credentials__StoredCredential;
typedef struct Spotify__Login5__V3__Credentials__Password Spotify__Login5__V3__Credentials__Password;
typedef struct Spotify__Login5__V3__Credentials__FacebookAccessToken Spotify__Login5__V3__Credentials__FacebookAccessToken;
typedef struct Spotify__Login5__V3__Credentials__OneTimeToken Spotify__Login5__V3__Credentials__OneTimeToken;
typedef struct Spotify__Login5__V3__Credentials__ParentChildCredential Spotify__Login5__V3__Credentials__ParentChildCredential;
typedef struct Spotify__Login5__V3__Credentials__AppleSignInCredential Spotify__Login5__V3__Credentials__AppleSignInCredential;
typedef struct Spotify__Login5__V3__Credentials__SamsungSignInCredential Spotify__Login5__V3__Credentials__SamsungSignInCredential;
typedef struct Spotify__Login5__V3__Credentials__GoogleSignInCredential Spotify__Login5__V3__Credentials__GoogleSignInCredential;


/* --- enums --- */


/* --- messages --- */

struct  Spotify__Login5__V3__Credentials__StoredCredential
{
  ProtobufCMessage base;
  char *username;
  ProtobufCBinaryData data;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__STORED_CREDENTIAL__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__stored_credential__descriptor) \
    , (char *)protobuf_c_empty_string, {0,NULL} }


struct  Spotify__Login5__V3__Credentials__Password
{
  ProtobufCMessage base;
  char *id;
  char *password;
  ProtobufCBinaryData padding;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__PASSWORD__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__password__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, {0,NULL} }


struct  Spotify__Login5__V3__Credentials__FacebookAccessToken
{
  ProtobufCMessage base;
  char *fb_uid;
  char *access_token;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__FACEBOOK_ACCESS_TOKEN__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__facebook_access_token__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


struct  Spotify__Login5__V3__Credentials__OneTimeToken
{
  ProtobufCMessage base;
  char *token;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__ONE_TIME_TOKEN__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__one_time_token__descriptor) \
    , (char *)protobuf_c_empty_string }


struct  Spotify__Login5__V3__Credentials__ParentChildCredential
{
  ProtobufCMessage base;
  char *child_id;
  Spotify__Login5__V3__Credentials__StoredCredential *parent_stored_credential;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__PARENT_CHILD_CREDENTIAL__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__parent_child_credential__descriptor) \
    , (char *)protobuf_c_empty_string, NULL }


struct  Spotify__Login5__V3__Credentials__AppleSignInCredential
{
  ProtobufCMessage base;
  char *auth_code;
  char *redirect_uri;
  char *bundle_id;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__APPLE_SIGN_IN_CREDENTIAL__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__apple_sign_in_credential__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


struct  Spotify__Login5__V3__Credentials__SamsungSignInCredential
{
  ProtobufCMessage base;
  char *auth_code;
  char *redirect_uri;
  char *id_token;
  char *token_endpoint_url;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__SAMSUNG_SIGN_IN_CREDENTIAL__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


struct  Spotify__Login5__V3__Credentials__GoogleSignInCredential
{
  ProtobufCMessage base;
  char *auth_code;
  char *redirect_uri;
};
#define SPOTIFY__LOGIN5__V3__CREDENTIALS__GOOGLE_SIGN_IN_CREDENTIAL__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__credentials__google_sign_in_credential__descriptor) \
    , (char *)protobuf_c_empty_string, (char *)protobuf_c_empty_string }


/* Spotify__Login5__V3__Credentials__StoredCredential methods */
void   spotify__login5__v3__credentials__stored_credential__init
                     (Spotify__Login5__V3__Credentials__StoredCredential         *message);
size_t spotify__login5__v3__credentials__stored_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__StoredCredential   *message);
size_t spotify__login5__v3__credentials__stored_credential__pack
                     (const Spotify__Login5__V3__Credentials__StoredCredential   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__stored_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__StoredCredential   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__StoredCredential *
       spotify__login5__v3__credentials__stored_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__stored_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__StoredCredential *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__Password methods */
void   spotify__login5__v3__credentials__password__init
                     (Spotify__Login5__V3__Credentials__Password         *message);
size_t spotify__login5__v3__credentials__password__get_packed_size
                     (const Spotify__Login5__V3__Credentials__Password   *message);
size_t spotify__login5__v3__credentials__password__pack
                     (const Spotify__Login5__V3__Credentials__Password   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__password__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__Password   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__Password *
       spotify__login5__v3__credentials__password__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__password__free_unpacked
                     (Spotify__Login5__V3__Credentials__Password *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__FacebookAccessToken methods */
void   spotify__login5__v3__credentials__facebook_access_token__init
                     (Spotify__Login5__V3__Credentials__FacebookAccessToken         *message);
size_t spotify__login5__v3__credentials__facebook_access_token__get_packed_size
                     (const Spotify__Login5__V3__Credentials__FacebookAccessToken   *message);
size_t spotify__login5__v3__credentials__facebook_access_token__pack
                     (const Spotify__Login5__V3__Credentials__FacebookAccessToken   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__facebook_access_token__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__FacebookAccessToken   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__FacebookAccessToken *
       spotify__login5__v3__credentials__facebook_access_token__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__facebook_access_token__free_unpacked
                     (Spotify__Login5__V3__Credentials__FacebookAccessToken *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__OneTimeToken methods */
void   spotify__login5__v3__credentials__one_time_token__init
                     (Spotify__Login5__V3__Credentials__OneTimeToken         *message);
size_t spotify__login5__v3__credentials__one_time_token__get_packed_size
                     (const Spotify__Login5__V3__Credentials__OneTimeToken   *message);
size_t spotify__login5__v3__credentials__one_time_token__pack
                     (const Spotify__Login5__V3__Credentials__OneTimeToken   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__one_time_token__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__OneTimeToken   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__OneTimeToken *
       spotify__login5__v3__credentials__one_time_token__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__one_time_token__free_unpacked
                     (Spotify__Login5__V3__Credentials__OneTimeToken *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__ParentChildCredential methods */
void   spotify__login5__v3__credentials__parent_child_credential__init
                     (Spotify__Login5__V3__Credentials__ParentChildCredential         *message);
size_t spotify__login5__v3__credentials__parent_child_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__ParentChildCredential   *message);
size_t spotify__login5__v3__credentials__parent_child_credential__pack
                     (const Spotify__Login5__V3__Credentials__ParentChildCredential   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__parent_child_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__ParentChildCredential   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__ParentChildCredential *
       spotify__login5__v3__credentials__parent_child_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__parent_child_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__ParentChildCredential *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__AppleSignInCredential methods */
void   spotify__login5__v3__credentials__apple_sign_in_credential__init
                     (Spotify__Login5__V3__Credentials__AppleSignInCredential         *message);
size_t spotify__login5__v3__credentials__apple_sign_in_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__AppleSignInCredential   *message);
size_t spotify__login5__v3__credentials__apple_sign_in_credential__pack
                     (const Spotify__Login5__V3__Credentials__AppleSignInCredential   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__apple_sign_in_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__AppleSignInCredential   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__AppleSignInCredential *
       spotify__login5__v3__credentials__apple_sign_in_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__apple_sign_in_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__AppleSignInCredential *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__SamsungSignInCredential methods */
void   spotify__login5__v3__credentials__samsung_sign_in_credential__init
                     (Spotify__Login5__V3__Credentials__SamsungSignInCredential         *message);
size_t spotify__login5__v3__credentials__samsung_sign_in_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__SamsungSignInCredential   *message);
size_t spotify__login5__v3__credentials__samsung_sign_in_credential__pack
                     (const Spotify__Login5__V3__Credentials__SamsungSignInCredential   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__samsung_sign_in_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__SamsungSignInCredential   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__SamsungSignInCredential *
       spotify__login5__v3__credentials__samsung_sign_in_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__samsung_sign_in_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__SamsungSignInCredential *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Credentials__GoogleSignInCredential methods */
void   spotify__login5__v3__credentials__google_sign_in_credential__init
                     (Spotify__Login5__V3__Credentials__GoogleSignInCredential         *message);
size_t spotify__login5__v3__credentials__google_sign_in_credential__get_packed_size
                     (const Spotify__Login5__V3__Credentials__GoogleSignInCredential   *message);
size_t spotify__login5__v3__credentials__google_sign_in_credential__pack
                     (const Spotify__Login5__V3__Credentials__GoogleSignInCredential   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__credentials__google_sign_in_credential__pack_to_buffer
                     (const Spotify__Login5__V3__Credentials__GoogleSignInCredential   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Credentials__GoogleSignInCredential *
       spotify__login5__v3__credentials__google_sign_in_credential__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__credentials__google_sign_in_credential__free_unpacked
                     (Spotify__Login5__V3__Credentials__GoogleSignInCredential *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Login5__V3__Credentials__StoredCredential_Closure)
                 (const Spotify__Login5__V3__Credentials__StoredCredential *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__Password_Closure)
                 (const Spotify__Login5__V3__Credentials__Password *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__FacebookAccessToken_Closure)
                 (const Spotify__Login5__V3__Credentials__FacebookAccessToken *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__OneTimeToken_Closure)
                 (const Spotify__Login5__V3__Credentials__OneTimeToken *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__ParentChildCredential_Closure)
                 (const Spotify__Login5__V3__Credentials__ParentChildCredential *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__AppleSignInCredential_Closure)
                 (const Spotify__Login5__V3__Credentials__AppleSignInCredential *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__SamsungSignInCredential_Closure)
                 (const Spotify__Login5__V3__Credentials__SamsungSignInCredential *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Credentials__GoogleSignInCredential_Closure)
                 (const Spotify__Login5__V3__Credentials__GoogleSignInCredential *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__stored_credential__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__password__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__facebook_access_token__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__one_time_token__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__parent_child_credential__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__apple_sign_in_credential__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__samsung_sign_in_credential__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__credentials__google_sign_in_credential__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_login5_5fcredentials_2eproto__INCLUDED */
